@@ -1,6 +1,6 @@
 /*
  * antiword.h
- * Copyright (C) 1998-2002 A.J. van Os; Released under GPL
+ * Copyright (C) 1998-2003 A.J. van Os; Released under GNU GPL
  *
  * Description:
  * Generic include file for project 'Antiword'
@@ -42,13 +42,31 @@
  #endif /* __riscos */
 #endif /* !PATH_MAX */
 
+#if !defined(CHAR_BIT)
+#define CHAR_BIT		8
+#endif /* CHAR_BIT */
+
+#if !defined(TIME_T_MIN)
+#define TIME_T_MIN		((time_t)0 < (time_t)-1 ?\
+				(time_t)0 :\
+				(time_t)1 << (sizeof(time_t) * CHAR_BIT - 1))
+#endif /* TIMER_T_MIN */
+
+#if !defined(TIME_T_MAX)
+#if defined(__TURBOC__)	/* Turbo C chokes on the subtraction below */
+#define TIME_T_MAX		(LONG_MAX)
+#else	/* All others */
+#define TIME_T_MAX		(~(time_t)0 - TIME_T_MIN)
+#endif /* __TURBOC__ */
+#endif /* TIME_T_MAX */
+
 #if !defined(SIZE_T_MAX)
-#define SIZE_T_MAX		(size_t)UINT_MAX
+#define SIZE_T_MAX		(~(size_t)0)
 #endif /* SIZE_T_MAX */
 
 #if defined(__riscos)
 #define FILE_SEPARATOR		"."
-#elif defined(__dos)
+#elif defined(__dos) || defined(__CYGMING__)
 #define FILE_SEPARATOR		"\\"
 #else	/* All others */
 #define FILE_SEPARATOR		"/"
@@ -120,9 +138,15 @@
 /* Macros */
 #define STREQ(x,y)	(*(x) == *(y) && strcmp(x,y) == 0)
 #define STRNEQ(x,y,n)	(*(x) == *(y) && strncmp(x,y,n) == 0)
+#if defined(__dos)
+#define STRCEQ(x,y)	(stricmp(x,y) == 0)
+#else
+#define STRCEQ(x,y)	(strcasecmp(x,y) == 0)
+#endif /* __dos */
 #define elementsof(a)	(sizeof(a) / sizeof(a[0]))
 #define odd(x)		(((x)&0x01)!=0)
 #define ROUND4(x)	(((x)+3)&~0x03)
+#define ROUND128(x)	(((x)+127)&~0x7f)
 #define BIT(x)		(1UL << (x))
 #if !defined(max)
 #define max(x,y)	((x)>(y)?(x):(y))
@@ -172,10 +196,22 @@
 #define GLOBAL_ANTIWORD_DIR	"SYS:.antiword"
 #define ANTIWORD_DIR		".antiword"
 #define FONTNAMES_FILE		"fontnames"
+#elif defined(N_PLAT_NLM)
+#define GLOBAL_ANTIWORD_DIR	"SYS:/antiword"
+#define ANTIWORD_DIR		"antiword"
+#define FONTNAMES_FILE		"fontname.txt"
 #elif defined(__vms)
 #define GLOBAL_ANTIWORD_DIR	"/usr/share/antiword"
 #define ANTIWORD_DIR		"antiword"
 #define FONTNAMES_FILE		"fontnames"
+#elif defined(__BEOS__)
+#define GLOBAL_ANTIWORD_DIR	"/boot/home/config/apps/antiword"
+#define ANTIWORD_DIR		"antiword"
+#define FONTNAMES_FILE		"fontnames"
+#elif defined(__CYGMING__)
+#define GLOBAL_ANTIWORD_DIR	"C:\\antiword"
+#define ANTIWORD_DIR		"antiword"
+#define FONTNAMES_FILE		"fontnames"
 #else	/* All others */
 #define GLOBAL_ANTIWORD_DIR	"/usr/share/antiword"
 #define ANTIWORD_DIR		".antiword"
@@ -191,12 +227,15 @@
 
 /* asc85enc.c */
 extern void	vASCII85EncodeByte(FILE *, int);
-extern void	vASCII85EncodeArray(FILE *, FILE *, int);
-extern void	vASCII85EncodeFile(FILE *, FILE *, int);
+extern void	vASCII85EncodeArray(FILE *, FILE *, size_t);
+extern void	vASCII85EncodeFile(FILE *, FILE *, size_t);
 /* blocklist.c */
 extern void	vDestroyTextBlockList(void);
-extern BOOL	bAdd2TextBlockList(text_block_type *);
-extern void	vSplitBlockList(ULONG, ULONG, ULONG, ULONG, ULONG, BOOL);
+extern BOOL	bAdd2TextBlockList(const text_block_type *);
+extern void	vSplitBlockList(FILE *, ULONG, ULONG, ULONG, ULONG, ULONG,
+			ULONG, ULONG, ULONG, BOOL);
+extern BOOL	bExistsTextBox(void);
+extern BOOL	bExistsHdrTextBox(void);
 extern USHORT	usNextChar(FILE *, list_id_enum, ULONG *, ULONG *, USHORT *);
 extern ULONG	ulCharPos2FileOffset(ULONG);
 extern ULONG	ulGetSeqNumber(ULONG);
@@ -206,18 +245,20 @@ extern ULONG	ulGetDocumentLength(void);
 /* chartrans.c */
 extern UCHAR	ucGetNbspValue(void);
 extern BOOL	bReadCharacterMappingTable(const char *);
-extern ULONG	ulTranslateCharacters(USHORT, ULONG, int, encoding_type, BOOL);
+extern ULONG	ulTranslateCharacters(USHORT, ULONG, int, conversion_type,
+			encoding_type, BOOL);
 extern ULONG	ulToUpper(ULONG);
 /* datalist.c */
 extern void	vDestroyDataBlockList(void);
-extern BOOL	bAdd2DataBlockList(data_block_type *);
+extern BOOL	bAdd2DataBlockList(const data_block_type *);
+extern ULONG	ulGetDataOffset(FILE *);
 extern BOOL	bSetDataOffset(FILE *, ULONG);
 extern int	iNextByte(FILE *);
 extern USHORT	usNextWord(FILE *);
 extern ULONG	ulNextLong(FILE *);
 extern USHORT	usNextWordBE(FILE *);
 extern ULONG	ulNextLongBE(FILE *);
-extern int	iSkipBytes(FILE *, size_t);
+extern size_t	tSkipBytes(FILE *, size_t);
 extern ULONG	ulDataPos2FileOffset(ULONG);
 /* depot.c */
 extern void	vDestroySmallBlockList(void);
@@ -226,19 +267,25 @@ extern ULONG	ulDepotOffset(ULONG, size_t);
 /* dib2eps & dib2sprt.c */
 extern BOOL	bTranslateDIB(diagram_type *,
 			FILE *, ULONG, const imagedata_type *);
-/* draw.c & postscript.c */
+/* draw.c & output.c */
 extern BOOL	bAddDummyImage(diagram_type *, const imagedata_type *);
 extern diagram_type *pCreateDiagram(const char *, const char *);
-extern void	vAddFonts2Diagram(diagram_type *);
+extern void	vPrologue2(diagram_type *, int);
+extern void	vMove2NextLine(diagram_type *, draw_fontref, USHORT);
 extern void	vSubstring2Diagram(diagram_type *,
-			char *, int, long, int, UCHAR,
-			draw_fontref, short, short);
-extern void	vMove2NextLine(diagram_type *, draw_fontref, short);
-extern void	vStartOfParagraph2Diagram(diagram_type *,
-			draw_fontref, short, long);
-extern void	vEndOfParagraph2Diagram(diagram_type *,
-			draw_fontref, short, long);
-extern void	vEndOfPage2Diagram(diagram_type *, draw_fontref, short, long);
+			char *, size_t, long, UCHAR, USHORT,
+			draw_fontref, USHORT, USHORT);
+extern void	vStartOfParagraph1(diagram_type *, long);
+extern void	vStartOfParagraph2(diagram_type *);
+extern void	vEndOfParagraph(diagram_type *, draw_fontref, USHORT, long);
+extern void	vEndOfPage(diagram_type *, long);
+extern void	vSetHeaders(diagram_type *, USHORT);
+extern void	vStartOfList(diagram_type *, UCHAR, BOOL);
+extern void	vEndOfList(diagram_type *);
+extern void	vStartOfListItem(diagram_type *, BOOL);
+extern void	vEndOfTable(diagram_type *);
+extern BOOL	bAddTableRow(diagram_type *, char **, int,
+			const short *, UCHAR);
 #if defined(__riscos)
 extern void	vImage2Diagram(diagram_type *, const imagedata_type *,
 			UCHAR *, size_t);
@@ -272,10 +319,11 @@ extern void	vAdd2FontInfoList(const font_block_type *);
 extern const font_block_type	*pGetNextFontInfoListItem(
 					const font_block_type *);
 /* fonts.c */
-extern int	iGetFontByNumber(UCHAR, UCHAR);
+extern int	iGetFontByNumber(UCHAR, USHORT);
 extern const char	*szGetOurFontname(int);
-extern int	iFontname2Fontnumber(const char *, UCHAR);
-extern void	vCreate2FontTable(FILE *, const UCHAR *);
+extern int	iFontname2Fontnumber(const char *, USHORT);
+extern void	vCreate0FontTable(void);
+extern void	vCreate2FontTable(FILE *, int, const UCHAR *);
 extern void	vCreate6FontTable(FILE *, ULONG,
 			const ULONG *, size_t, const UCHAR *);
 extern void	vCreate8FontTable(FILE *, const pps_info_type *,
@@ -288,9 +336,11 @@ extern size_t	tGetFontTableLength(void);
 /* fonts_r.c & fonts_u.c */
 extern FILE	*pOpenFontTableFile(void);
 extern void	vCloseFont(void);
-extern draw_fontref	tOpenFont(UCHAR, UCHAR, short);
-extern draw_fontref	tOpenTableFont(short);
-extern long	lComputeStringWidth(char *, int, draw_fontref, short);
+extern draw_fontref	tOpenFont(UCHAR, USHORT, USHORT);
+extern draw_fontref	tOpenTableFont(USHORT);
+extern long	lComputeStringWidth(const char *, size_t, draw_fontref, USHORT);
+extern size_t	tCountColumns(const char *, size_t);
+extern size_t	tGetCharacterLength(const char *);
 /* fonts_u.c */
 #if !defined(__riscos)
 extern const char	*szGetFontname(draw_fontref);
@@ -309,10 +359,18 @@ extern BOOL	bTranslateImage(diagram_type *,
 			FILE *, BOOL, ULONG, const imagedata_type *);
 /* jpeg2eps.c & jpeg2spr.c */
 extern BOOL	bTranslateJPEG(diagram_type *,
-			FILE *, ULONG, int, const imagedata_type *);
+			FILE *, ULONG, size_t, const imagedata_type *);
+/* listlist.c */
+extern void	vDestroyListInfoList(void);
+extern void	vBuildLfoList(const UCHAR *, size_t);
+extern void	vAdd2ListInfoList(ULONG, USHORT, UCHAR,
+			const list_block_type *);
+extern const list_block_type	*pGetListInfo(USHORT, UCHAR);
+extern USHORT	usGetListValue(int, int, const style_block_type *);
 /* misc.c */
 #if !defined(__riscos)
 extern const char	*szGetHomeDirectory(void);
+extern const char	*szGetAntiwordDirectory(void);
 #endif /* !__riscos */
 extern long	lGetFilesize(const char *);
 #if defined(DEBUG)
@@ -323,14 +381,17 @@ extern BOOL	bCheckDoubleLinkedList(output_type *);
 extern BOOL	bReadBytes(UCHAR *, size_t, ULONG, FILE *);
 extern BOOL	bReadBuffer(FILE *, ULONG, const ULONG *, size_t, size_t,
 			UCHAR *, ULONG, size_t);
-extern ULONG	ulColor2Color(int);
+extern ULONG	ulColor2Color(UCHAR);
 extern output_type *pSplitList(output_type *);
-extern int	iNumber2Roman(UINT, BOOL, char *);
-extern int	iNumber2Alpha(UINT, BOOL, char *);
+extern size_t	tNumber2Roman(UINT, BOOL, char *);
+extern size_t	tNumber2Alpha(UINT, BOOL, char *);
 extern char	*unincpy(char *, const UCHAR *, size_t);
 extern size_t	unilen(const UCHAR *);
 extern const char	*szBasename(const char *);
-extern long	lComputeLeading(short);
+extern long	lComputeLeading(USHORT);
+extern size_t	tUcs2Utf8(ULONG, char *, size_t);
+extern void	vGetBulletValue(conversion_type, encoding_type, char *, size_t);
+extern BOOL	bAllZero(const UCHAR *, size_t);
 /* notes.c */
 extern void	vDestroyNotesInfoLists(void);
 extern void	vGetNotesInfo(FILE *, const pps_info_type *,
@@ -352,7 +413,7 @@ extern void	vAlign2Window(diagram_type *, output_type *,
 extern void	vJustify2Window(diagram_type *, output_type *,
 			long, long, UCHAR);
 extern void	vResetStyles(void);
-extern int	iStyle2Window(char *, const style_block_type *,
+extern size_t	tStyle2Window(char *, const style_block_type *,
 			const section_block_type *);
 extern void	vTableRow2Window(diagram_type *,
 			output_type *, const row_block_type *);
@@ -362,7 +423,26 @@ extern void	vAdd2PictInfoList(const picture_block_type *);
 extern ULONG	ulGetPictInfoListItem(ULONG);
 /* png2eps.c & png2spr.c */
 extern BOOL	bTranslatePNG(diagram_type *,
-			FILE *, ULONG, int, const imagedata_type *);
+			FILE *, ULONG, size_t, const imagedata_type *);
+/* postscript.c */
+extern void	vProloguePS(diagram_type *,
+			const char *, const char *, const options_type *);
+extern void	vEpiloguePS(diagram_type *);
+extern void	vImageProloguePS(diagram_type *, const imagedata_type *);
+extern void	vImageEpiloguePS(diagram_type *);
+extern BOOL	bAddDummyImagePS(diagram_type *, const imagedata_type *);
+extern void	vAddFontsPS(diagram_type *);
+extern void	vMove2NextLinePS(diagram_type *, USHORT);
+extern void	vSubstringPS(diagram_type *,
+				char *, size_t, long, UCHAR, USHORT,
+				draw_fontref, USHORT, USHORT);
+extern void	vStartOfParagraphPS(diagram_type *, long);
+extern void	vEndOfParagraphPS(diagram_type *, draw_fontref, USHORT, long);
+extern void	vEndOfPagePS(diagram_type *);
+/* prop0.c */
+extern void	vGet0SepInfo(FILE *, const UCHAR *);
+extern void	vGet0PapInfo(FILE *, const UCHAR *);
+extern void	vGet0ChrInfo(FILE *, const UCHAR *);
 /* prop2.c */
 extern void	vGet2SepInfo(FILE *, const UCHAR *);
 extern row_info_enum	eGet2RowInfo(int,
@@ -396,6 +476,9 @@ extern row_info_enum	eGet8RowInfo(int,
 			const UCHAR *, int, row_block_type *);
 extern void	vGet8StyleInfo(int,
 			const UCHAR *, int, style_block_type *);
+extern void	vGet8LstInfo(FILE *, const pps_info_type *,
+			const ULONG *, size_t, const ULONG *, size_t,
+			const UCHAR *);
 extern void	vGet8PapInfo(FILE *, const pps_info_type *,
 			const ULONG *, size_t, const ULONG *, size_t,
 			const UCHAR *);
@@ -412,7 +495,7 @@ extern row_info_enum	ePropMod2RowInfo(USHORT, int);
 /* propmod.c */
 extern void	vDestroyPropModList(void);
 extern void	vAdd2PropModList(const UCHAR *);
-extern const UCHAR	*pucReadPropModListItem(USHORT);
+extern const UCHAR	*aucReadPropModListItem(USHORT);
 /* rowlist.c */
 extern void	vDestroyRowInfoList(void);
 extern void	vAdd2RowInfoList(const row_block_type *);
@@ -422,7 +505,7 @@ extern const row_block_type	*pGetNextRowInfoListItem(void);
 extern int	iGetFiletype(const char *);
 extern void	vSetFiletype(const char *, int);
 extern BOOL	bMakeDirectory(const char *);
-extern BOOL	bISO_8859_1_IsCurrent(void);
+extern int	iReadCurrentAlphabetNumber(void);
 extern int	iGetRiscOsVersion(void);
 extern BOOL	bDrawRenderDiag360(draw_diag *,
 			draw_redrawstr *, double, draw_error *);
@@ -450,6 +533,7 @@ extern void	vAdd2StyleInfoList(const style_block_type *);
 extern const style_block_type	*pGetNextStyleInfoListItem(
 					const style_block_type *);
 extern USHORT	usGetIstd(ULONG);
+extern BOOL	bStyleImpliesList(const style_block_type *, int);
 /* stylesheet.c */
 extern void	vDestroyStylesheetList(void);
 extern USHORT	usStc2istd(UCHAR);
@@ -461,36 +545,64 @@ extern void	vGet8Stylesheet(FILE *, const pps_info_type *,
 			const UCHAR *);
 extern void	vFillStyleFromStylesheet(USHORT, style_block_type *);
 extern void	vFillFontFromStylesheet(USHORT, font_block_type *);
+/* summary.c */
+extern void	vDestroySummaryInfo(void);
+extern void	vSet0SummaryInfo(FILE *, const UCHAR *);
+extern void	vSet2SummaryInfo(FILE *, int, const UCHAR *);
+extern void	vSet6SummaryInfo(FILE *, const pps_info_type *,
+			const ULONG *, size_t, const ULONG *, size_t,
+			const UCHAR *);
+extern void	vSet8SummaryInfo(FILE *, const pps_info_type *,
+			const ULONG *, size_t, const ULONG *, size_t,
+			const UCHAR *);
+extern const char	*szGetTitle(void);
+extern const char	*szGetSubject(void);
+extern const char	*szGetAuthor(void);
+extern const char	*szGetLastSaveDtm(void);
+extern const char	*szGetCompany(void);
+extern const char	*szGetLanguage(void);
 /* tabstop.c */
 extern void	vSetDefaultTabWidth(FILE *, const pps_info_type *,
 			const ULONG *, size_t, const ULONG *, size_t,
 			const UCHAR *, int);
 extern long	lGetDefaultTabWidth(void);
+/* text.c */
+extern void	vPrologueTXT(diagram_type *, const options_type *);
+extern void	vEpilogueTXT(FILE *);
+extern void	vMove2NextLineTXT(diagram_type *);
+extern void	vSubstringTXT(diagram_type *, const char *, size_t, long);
+extern void	vStartOfParagraphTXT(diagram_type *, long);
+extern void	vEndOfParagraphTXT(diagram_type *, long);
+extern void	vEndOfPageTXT(diagram_type *, long);
 /* unix.c */
 #if !defined(__riscos)
 extern void	werr(int, const char *, ...);
 extern void	visdelay_begin(void);
 extern void	visdelay_end(void);
 #endif /* !__riscos */
-/* utf-8.c */
+/* utf8.c */
 #if !defined(__riscos)
-extern long	utf8_strwidth(const char *, int);
+extern long	utf8_strwidth(const char *, size_t);
+extern int	utf8_chrlength(const char *);
 extern BOOL	is_locale_utf8(void);
 #endif /* !__riscos */
 /* word2text.c */
-extern BOOL	bWord2Text(FILE *, long, diagram_type *);
+extern BOOL	bWordDecryptor(FILE *, long, diagram_type *);
 /* worddos.c */
 extern int	iInitDocumentDOS(FILE *, long);
 /* wordlib.c */
 extern BOOL	bIsWordForDosFile(FILE *, long);
 extern BOOL	bIsRtfFile(FILE *);
+extern BOOL	bIsWordPerfectFile(FILE *);
 extern BOOL	bIsWinWord12File(FILE *, long);
 extern BOOL	bIsMacWord45File(FILE *);
 extern int	iGuessVersionNumber(FILE *, long);
 extern int	iGetVersionNumber(const UCHAR *);
-extern BOOL	bIsWord6MacFile(void);
+extern BOOL	bIsOldMacFile(void);
 extern int	iInitDocument(FILE *, long);
 extern void	vFreeDocument(void);
+/* wordmac.c */
+extern int	iInitDocumentMAC(FILE *, long);
 /* wordole.c */
 extern int	iInitDocumentOLE(FILE *, long);
 /* wordwin.c */
@@ -501,6 +613,24 @@ extern void	*xcalloc(size_t, size_t);
 extern void 	*xrealloc(void *, size_t);
 extern char	*xstrdup(const char *);
 extern void 	*xfree(void *);
+/* xml.c */
+extern void	vCreateBookIntro(diagram_type *, int, encoding_type);
+extern void	vPrologueXML(diagram_type *);
+extern void	vEpilogueXML(diagram_type *);
+extern void	vMove2NextLineXML(diagram_type *);
+extern void	vSubstringXML(diagram_type *,
+				const char *, size_t, long, USHORT);
+extern void	vStartOfParagraphXML(diagram_type *, UINT);
+extern void	vEndOfParagraphXML(diagram_type *, UINT);
+extern void	vEndOfPageXML(diagram_type *);
+extern void	vSetHeadersXML(diagram_type *, USHORT);
+extern void	vStartOfListXML(diagram_type *, UCHAR, BOOL);
+extern void	vEndOfListXML(diagram_type *);
+extern void	vStartOfListItemXML(diagram_type *, BOOL);
+extern void	vEndOfTableXML(diagram_type *);
+extern void	vAddTableRowXML(diagram_type *, char **, int,
+			const short *, UCHAR);
+
 
 /* For use with Gray Watson dmalloc library */
 #if defined(DMALLOC)
