@@ -1,6 +1,6 @@
 /*
  * draw.c
- * Copyright (C) 1998-2001 A.J. van Os; Released under GPL
+ * Copyright (C) 1998-2003 A.J. van Os; Released under GPL
  *
  * Description:
  * Functions to deal with the Draw format
@@ -17,6 +17,7 @@
 #include "win.h"
 #include "antiword.h"
 
+
 /* The work area must be a little bit larger than the diagram */
 #define WORKAREA_EXTENSION	    5
 /* Diagram memory */
@@ -30,9 +31,9 @@ static BOOL	(*bDrawRenderDiag)(draw_diag *,
 			draw_redrawstr *, double, draw_error *) = NULL;
 
 /*
- * vCreateMainWindow - create the Main Window
+ * vCreateMainWindow - create the Main window
  *
- * remark: does not return if the Main Window can't be created
+ * remark: does not return if the Main window can't be created
  */
 static wimp_w
 tCreateMainWindow(void)
@@ -78,9 +79,9 @@ tCreateMainWindow(void)
 } /* end of tCreateMainWindow */
 
 /*
- * vCreateScaleWindow - create the Scale view Window
+ * vCreateScaleWindow - create the Scale view window
  *
- * remark: does not return if the Scale view Window can't be created
+ * remark: does not return if the Scale view window can't be created
  */
 static wimp_w
 tCreateScaleWindow(void)
@@ -115,6 +116,7 @@ pCreateDiagram(const char *szTask, const char *szFilename)
 
 	/* Create the main window */
 	tMainWindow = tCreateMainWindow();
+
 	/* Create the scale view window */
 	tScaleWindow = tCreateScaleWindow();
 
@@ -161,14 +163,26 @@ pCreateDiagram(const char *szTask, const char *szFilename)
 static void
 vDestroyDiagram(wimp_w tWindow, diagram_type *pDiag)
 {
+	wimp_wstate	tWindowState;
+
 	DBG_MSG("vDestroyDiagram");
 
 	fail(pDiag != NULL && pDiag->tMainWindow != tWindow);
 
+	/* Close the main window */
 	wimpt_noerr(wimp_close_wind(tWindow));
 	if (pDiag == NULL) {
 		return;
 	}
+
+	/* Close the scale window if it's open */
+	wimpt_noerr(wimp_get_wind_state(pDiag->tScaleWindow, &tWindowState));
+	if ((tWindowState.flags & wimp_WOPEN) == wimp_WOPEN) {
+		DBG_MSG("Close scale window");
+		wimpt_noerr(wimp_close_wind(pDiag->tScaleWindow));
+	}
+
+	/* Free the memory */
 	if (pDiag->tInfo.data != NULL && pDiag->tMemorySize != 0) {
 		flex_free((flex_ptr)&pDiag->tInfo.data);
 	}
@@ -223,10 +237,10 @@ vExtendDiagramSize(diagram_type *pDiag, size_t tSize)
 } /* end of vExtendDiagramSize */
 
 /*
- * vAddFonts2Diagram - add a font list to a diagram
+ * vPrologue2 - prologue part 2; add a font list to a diagram
  */
 void
-vAddFonts2Diagram(diagram_type *pDiag)
+vPrologue2(diagram_type *pDiag, int iWordVersion)
 {
 	draw_objectType	tNew;
 	draw_error	tError;
@@ -269,31 +283,31 @@ vAddFonts2Diagram(diagram_type *pDiag)
 		vPrintDrawError(&tError);
 	}
 	tNew.fontList = xfree(tNew.fontList);
-} /* end of vAddFonts2Diagram */
+} /* end of vPrologue2 */
 
 /*
  * vSubstring2Diagram - put a sub string into a diagram
  */
 void
 vSubstring2Diagram(diagram_type *pDiag,
-	char *szString, int iStringLength, long lStringWidth,
-	int iColor, unsigned char ucFontstyle, draw_fontref tFontRef,
-	int iFontsize, int iMaxFontsize)
+	char *szString, size_t tStringLength, long lStringWidth,
+	UCHAR ucFontColor, USHORT usFontstyle, draw_fontref tFontRef,
+	USHORT usFontSize, USHORT usMaxFontSize)
 {
 	draw_objectType	tNew;
 	draw_error	tError;
 	draw_object	tHandle;
-	long	lSizeX, lSizeY, lOffset, l20;
+	long	lSizeX, lSizeY, lOffset, l20, lYMove;
 	size_t	tRealSize, tSize;
 
 	fail(pDiag == NULL || szString == NULL);
 	fail(pDiag->lXleft < 0);
-	fail(iStringLength != strlen(szString));
-	fail(iFontsize < MIN_FONT_SIZE || iFontsize > MAX_FONT_SIZE);
-	fail(iMaxFontsize < MIN_FONT_SIZE || iMaxFontsize > MAX_FONT_SIZE);
-	fail(iFontsize > iMaxFontsize);
+	fail(tStringLength != strlen(szString));
+	fail(usFontSize < MIN_FONT_SIZE || usFontSize > MAX_FONT_SIZE);
+	fail(usMaxFontSize < MIN_FONT_SIZE || usMaxFontSize > MAX_FONT_SIZE);
+	fail(usFontSize > usMaxFontSize);
 
-	if (szString[0] == '\0' || iStringLength <= 0) {
+	if (szString[0] == '\0' || tStringLength == 0) {
 		return;
 	}
 
@@ -303,23 +317,34 @@ vSubstring2Diagram(diagram_type *pDiag,
 		lSizeX = draw_screenToDraw(16);
 		lSizeY = draw_screenToDraw(32);
 	} else {
-		lOffset = lToBaseLine(iMaxFontsize);
-		l20 = lWord2DrawUnits20(iMaxFontsize);
-		lSizeX = lWord2DrawUnits00(iFontsize);
-		lSizeY = lWord2DrawUnits00(iFontsize);
+		lOffset = lToBaseLine(usMaxFontSize);
+		l20 = lWord2DrawUnits20(usMaxFontSize);
+		lSizeX = lWord2DrawUnits00(usFontSize);
+		lSizeY = lWord2DrawUnits00(usFontSize);
 	}
 
-	tRealSize = sizeof(draw_textstr) + iStringLength;
+	lYMove = 0;
+
+	/* Up for superscript */
+	if (bIsSuperscript(usFontstyle)) {
+		lYMove = lMilliPoints2DrawUnits((((long)usFontSize + 1) / 2) * 375);
+	}
+	/* Down for subscript */
+	if (bIsSubscript(usFontstyle)) {
+		lYMove = -lMilliPoints2DrawUnits((long)usFontSize * 125);
+	}
+
+	tRealSize = sizeof(draw_textstr) + tStringLength;
 	tSize = ROUND4(tRealSize);
 	vExtendDiagramSize(pDiag, tSize);
 	tNew.text = xmalloc(tSize);
 	tNew.text->tag = draw_OBJTEXT;
 	tNew.text->size = tSize;
 	tNew.text->bbox.x0 = (int)pDiag->lXleft;
-	tNew.text->bbox.y0 = (int)pDiag->lYtop;
+	tNew.text->bbox.y0 = (int)(pDiag->lYtop + lYMove);
 	tNew.text->bbox.x1 = (int)(pDiag->lXleft + lStringWidth);
-	tNew.text->bbox.y1 = (int)(pDiag->lYtop + l20);
-	tNew.text->textcolour = uiColor2Color(iColor);
+	tNew.text->bbox.y1 = (int)(pDiag->lYtop + l20 + lYMove);
+	tNew.text->textcolour = (draw_coltyp)ulColor2Color(ucFontColor);
 	tNew.text->background = 0xffffff00;	/* White */
 	tNew.text->textstyle.fontref = tFontRef;
 	tNew.text->textstyle.reserved8 = 0;
@@ -327,9 +352,9 @@ vSubstring2Diagram(diagram_type *pDiag,
 	tNew.text->fsizex = (int)lSizeX;
 	tNew.text->fsizey = (int)lSizeY;
 	tNew.text->coord.x = (int)pDiag->lXleft;
-	tNew.text->coord.y = (int)(pDiag->lYtop + lOffset);
-	strncpy(tNew.text->text, szString, iStringLength);
-	tNew.text->text[iStringLength] = '\0';
+	tNew.text->coord.y = (int)(pDiag->lYtop + lOffset + lYMove);
+	strncpy(tNew.text->text, szString, tStringLength);
+	tNew.text->text[tStringLength] = '\0';
 	memset((char *)tNew.text + tRealSize, 0, tSize - tRealSize);
 	if (!draw_createObject(&pDiag->tInfo, tNew, draw_LastObject,
 						TRUE, &tHandle, &tError)) {
@@ -346,7 +371,7 @@ vSubstring2Diagram(diagram_type *pDiag,
  */
 void
 vImage2Diagram(diagram_type *pDiag, const imagedata_type *pImg,
-	unsigned char *pucImage, size_t tImageSize)
+	UCHAR *pucImage, size_t tImageSize)
 {
   	draw_objectType	tTmp;
   	draw_imageType	tNew;
@@ -367,6 +392,8 @@ vImage2Diagram(diagram_type *pDiag, const imagedata_type *pImg,
 
 	lWidth = lPoints2DrawUnits(pImg->iHorSizeScaled);
 	lHeight = lPoints2DrawUnits(pImg->iVerSizeScaled);
+	DBG_DEC(lWidth);
+	DBG_DEC(lHeight);
 
 	pDiag->lYtop -= lHeight;
 
@@ -386,6 +413,9 @@ vImage2Diagram(diagram_type *pDiag, const imagedata_type *pImg,
 		memset((char *)tNew.sprite + tRealSize, 0, tSize - tRealSize);
 		break;
 	case imagetype_is_jpeg:
+#if defined(DEBUG)
+		(void)bGetJpegInfo(pucImage, tImageSize);
+#endif /* DEBUG */
 		tRealSize = sizeof(draw_jpegstrhdr) + tImageSize;
 		tSize = ROUND4(tRealSize);
 		vExtendDiagramSize(pDiag, tSize);
@@ -515,51 +545,170 @@ bAddDummyImage(diagram_type *pDiag, const imagedata_type *pImg)
  * vMove2NextLine - move to the next line
  */
 void
-vMove2NextLine(diagram_type *pDiag, draw_fontref tFontRef, int iFontsize)
+vMove2NextLine(diagram_type *pDiag, draw_fontref tFontRef, USHORT usFontSize)
 {
 	long	l20;
 
 	fail(pDiag == NULL);
-	fail(iFontsize < MIN_FONT_SIZE || iFontsize > MAX_FONT_SIZE);
+	fail(usFontSize < MIN_FONT_SIZE || usFontSize > MAX_FONT_SIZE);
 
 	if (tFontRef == 0) {
 		l20 = draw_screenToDraw(32 + 3);
 	} else {
-		l20 = lWord2DrawUnits20(iFontsize);
+		l20 = lWord2DrawUnits20(usFontSize);
 	}
 	pDiag->lYtop -= l20;
 } /* end of vMove2NextLine */
 
 /*
- * Create an end of paragraph by moving the y-high mark 1/3 line down
+ * Create an start of paragraph (Phase 1)
  */
 void
-vEndOfParagraph2Diagram(diagram_type *pDiag,
-			draw_fontref tFontRef, int iFontsize)
+vStartOfParagraph1(diagram_type *pDiag, long lBeforeIndentation)
 {
-	long	l20;
-
 	fail(pDiag == NULL);
-	fail(iFontsize < MIN_FONT_SIZE || iFontsize > MAX_FONT_SIZE);
+	fail(lBeforeIndentation < 0);
 
-	if (tFontRef == 0) {
-		l20 = draw_screenToDraw(32 + 3);
-	} else {
-		l20 = lWord2DrawUnits20(iFontsize);
-	}
-	pDiag->lYtop -= l20 / 3;	/* Line spacing */
 	pDiag->lXleft = 0;
-} /* end of vEndOfParagraph2Diagram */
+	pDiag->lYtop -= lMilliPoints2DrawUnits(lBeforeIndentation);
+} /* end of vStartOfParagraph1 */
+
+/*
+ * Create an start of paragraph (Phase 2)
+ */
+void
+vStartOfParagraph2(diagram_type *pDiag)
+{
+	/* DUMMY */
+} /* end of vStartOfParagraph2 */
+
+/*
+ * Create an end of paragraph
+ */
+void
+vEndOfParagraph(diagram_type *pDiag,
+	draw_fontref tFontRef, USHORT usFontSize, long lAfterIndentation)
+{
+	fail(pDiag == NULL);
+	fail(usFontSize < MIN_FONT_SIZE || usFontSize > MAX_FONT_SIZE);
+	fail(lAfterIndentation < 0);
+
+	pDiag->lXleft = 0;
+	pDiag->lYtop -= lMilliPoints2DrawUnits(lAfterIndentation);
+} /* end of vEndOfParagraph */
 
 /*
  * Create an end of page
  */
 void
-vEndOfPage2Diagram(diagram_type *pDiag,
-			draw_fontref tFontRef, int iFontsize)
+vEndOfPage(diagram_type *pDiag, long lAfterIndentation)
 {
-	vEndOfParagraph2Diagram(pDiag, tFontRef, iFontsize);
-} /* end of vEndOfPage2Diagram */
+	fail(pDiag == NULL);
+	fail(lAfterIndentation < 0);
+
+	pDiag->lXleft = 0;
+	pDiag->lYtop -= lMilliPoints2DrawUnits(lAfterIndentation);
+} /* end of vEndOfPage */
+
+/*
+ * vSetHeaders - set the headers
+ */
+void
+vSetHeaders(diagram_type *pDiag, USHORT usIstd)
+{
+	/* DUMMY */
+} /* end of vSetHeaders */
+
+/*
+ * Create a start of list
+ */
+void
+vStartOfList(diagram_type *pDiag, UCHAR ucNFC, BOOL bIsEndOfTable)
+{
+	/* DUMMY */
+} /* end of vStartOfList */
+
+/*
+ * Create an end of list
+ */
+void
+vEndOfList(diagram_type *pDiag)
+{
+	/* DUMMY */
+} /* end of vEndOfList */
+
+/*
+ * Create a start of a list item
+ */
+void
+vStartOfListItem(diagram_type *pDiag, BOOL bNoMarks)
+{
+	/* DUMMY */
+} /* end of vStartOfListItem */
+
+/*
+ * Create an end of a table
+ */
+void
+vEndOfTable(diagram_type *pDiag)
+{
+	/* DUMMY */
+} /* end of vEndTable */
+
+/*
+ * Add a table row
+ *
+ * Returns TRUE when conversion type is XML
+ */
+BOOL
+bAddTableRow(diagram_type *pDiag, char **aszColTxt,
+	int iNbrOfColumns, const short *asColumnWidth, UCHAR ucBorderInfo)
+{
+	/* DUMMY */
+	return FALSE;
+} /* end of bAddTableRow */
+
+/*
+ * vForceRedraw - force a redraw of the main window
+ */
+static void
+vForceRedraw(diagram_type *pDiag)
+{
+	wimp_wstate	tWindowState;
+	wimp_redrawstr	tRedraw;
+
+	DBG_MSG("vForceRedraw");
+
+	fail(pDiag == NULL);
+
+	DBG_DEC(pDiag->iScaleFactorCurr);
+
+	/* Read the size of the current diagram */
+	draw_queryBox(&pDiag->tInfo, (draw_box *)&tRedraw.box, TRUE);
+	tRedraw.w = pDiag->tMainWindow;
+	/* Adjust the size of the work area */
+	tRedraw.box.x0 = tRedraw.box.x0 * pDiag->iScaleFactorCurr / 100 - 1;
+	tRedraw.box.y0 = tRedraw.box.y0 * pDiag->iScaleFactorCurr / 100 - 1;
+	tRedraw.box.x1 = tRedraw.box.x1 * pDiag->iScaleFactorCurr / 100 + 1;
+	tRedraw.box.y1 = tRedraw.box.y1 * pDiag->iScaleFactorCurr / 100 + 1;
+	/* Work area extension */
+	tRedraw.box.x0 -= WORKAREA_EXTENSION;
+	tRedraw.box.y0 -= WORKAREA_EXTENSION;
+	tRedraw.box.x1 += WORKAREA_EXTENSION;
+	tRedraw.box.y1 += WORKAREA_EXTENSION;
+	wimpt_noerr(wimp_set_extent(&tRedraw));
+	/* Widen the box slightly to be sure all the edges are drawn */
+	tRedraw.box.x0 -= 5;
+	tRedraw.box.y0 -= 5;
+	tRedraw.box.x1 += 5;
+	tRedraw.box.y1 += 5;
+	/* Force the redraw */
+	wimpt_noerr(wimp_force_redraw(&tRedraw));
+	/* Reopen the window to show the correct size */
+	wimpt_noerr(wimp_get_wind_state(pDiag->tMainWindow, &tWindowState));
+	tWindowState.o.behind = -1;
+	wimpt_noerr(wimp_open_wind(&tWindowState.o));
+} /* end of vForceRedraw */
 
 /*
  * bVerifyDiagram - Verify the diagram generated from the Word file
@@ -571,9 +720,10 @@ bVerifyDiagram(diagram_type *pDiag)
 {
 	draw_error	tError;
 
-	fail(pDiag == NULL);
 	DBG_MSG("bVerifyDiagram");
 
+	fail(pDiag == NULL);
+
 	if (draw_verify_diag(&pDiag->tInfo, &tError)) {
 		return TRUE;
 	}
@@ -582,16 +732,19 @@ bVerifyDiagram(diagram_type *pDiag)
 	return FALSE;
 } /* end of bVerifyDiagram */
 
+/*
+ * vShowDiagram - put the diagram on the screen
+ */
 void
 vShowDiagram(diagram_type *pDiag)
 {
 	wimp_wstate	tWindowState;
 	wimp_redrawstr	tRedraw;
 
-	fail(pDiag == NULL);
-
 	DBG_MSG("vShowDiagram");
 
+	fail(pDiag == NULL);
+
 	wimpt_noerr(wimp_get_wind_state(pDiag->tMainWindow, &tWindowState));
 	tWindowState.o.behind = -1;
 	wimpt_noerr(wimp_open_wind(&tWindowState.o));
@@ -604,6 +757,7 @@ vShowDiagram(diagram_type *pDiag)
 	tRedraw.box.x1 += WORKAREA_EXTENSION;
 	tRedraw.box.y1 += WORKAREA_EXTENSION;
 	wimpt_noerr(wimp_set_extent(&tRedraw));
+	vForceRedraw(pDiag);
 } /* end of vShowDiagram */
 
 /*
@@ -794,48 +948,6 @@ vSetTitle(diagram_type *pDiag)
 	win_settitle(pDiag->tMainWindow, szTitle);
 } /* end of vSetTitle */
 
-/*
- * vForceRedraw - force a redraw of the main window
- */
-static void
-vForceRedraw(diagram_type *pDiag)
-{
-	wimp_wstate	tWindowState;
-	wimp_redrawstr	tRedraw;
-
-	DBG_MSG("vForceRedraw");
-
-	fail(pDiag == NULL);
-
-	DBG_DEC(pDiag->iScaleFactorCurr);
-
-	/* Read the size of the current diagram */
-	draw_queryBox(&pDiag->tInfo, (draw_box *)&tRedraw.box, TRUE);
-	tRedraw.w = pDiag->tMainWindow;
-	/* Adjust the size of the work area */
-	tRedraw.box.x0 = tRedraw.box.x0 * pDiag->iScaleFactorCurr / 100 - 1;
-	tRedraw.box.y0 = tRedraw.box.y0 * pDiag->iScaleFactorCurr / 100 - 1;
-	tRedraw.box.x1 = tRedraw.box.x1 * pDiag->iScaleFactorCurr / 100 + 1;
-	tRedraw.box.y1 = tRedraw.box.y1 * pDiag->iScaleFactorCurr / 100 + 1;
-	/* Work area extension */
-	tRedraw.box.x0 -= WORKAREA_EXTENSION;
-	tRedraw.box.y0 -= WORKAREA_EXTENSION;
-	tRedraw.box.x1 += WORKAREA_EXTENSION;
-	tRedraw.box.y1 += WORKAREA_EXTENSION;
-	wimpt_noerr(wimp_set_extent(&tRedraw));
-	/* Widen the box slightly to be sure all the edges are drawn */
-	tRedraw.box.x0 -= 5;
-	tRedraw.box.y0 -= 5;
-	tRedraw.box.x1 += 5;
-	tRedraw.box.y1 += 5;
-	/* Force the redraw */
-	wimpt_noerr(wimp_force_redraw(&tRedraw));
-	/* Reopen the window to show the correct size */
-	wimpt_noerr(wimp_get_wind_state(pDiag->tMainWindow, &tWindowState));
-	tWindowState.o.behind = -1;
-	wimpt_noerr(wimp_open_wind(&tWindowState.o));
-} /* end of vForceRedraw */
-
 /*
  * vScaleButtonClick - handle a mouse button click in the Scale view window
  */
@@ -890,6 +1002,9 @@ vScaleButtonClick(wimp_mousestr *m, diagram_type *pDiag)
 	}
 } /* end of vScaleButtonClick */
 
+/*
+ *
+ */
 static void
 vScaleKeyPressed(int chcode, wimp_caretstr *c, diagram_type *pDiag)
 {
