@@ -1,6 +1,6 @@
 /*
  * fonts.c
- * Copyright (C) 1998-2002 A.J. van Os; Released under GPL
+ * Copyright (C) 1998-2003 A.J. van Os; Released under GNU GPL
  *
  * Description:
  * Functions to deal with fonts (generic)
@@ -10,7 +10,7 @@
 #include <string.h>
 #include "antiword.h"
 
-/* Maximum line lenth in the font file */
+/* Maximum line length in the font file */
 #define FONT_LINE_LENGTH	81
 
 /* Pitch */
@@ -36,19 +36,19 @@ static font_table_type	*pFontTable = NULL;
  * returns the index into the FontTable, -1 if not found
  */
 int
-iGetFontByNumber(UCHAR ucWordFontnumber, UCHAR ucFontstyle)
+iGetFontByNumber(UCHAR ucWordFontNumber, USHORT usFontStyle)
 {
 	int	iIndex;
 
 	for (iIndex = 0; iIndex < (int)tFontTableRecords; iIndex++) {
-		if (ucWordFontnumber == pFontTable[iIndex].ucWordFontnumber &&
-		    ucFontstyle == pFontTable[iIndex].ucFontstyle &&
+		if (ucWordFontNumber == pFontTable[iIndex].ucWordFontNumber &&
+		    usFontStyle == pFontTable[iIndex].usFontStyle &&
 		    pFontTable[iIndex].szOurFontname[0] != '\0') {
 			return iIndex;
 		}
 	}
-	DBG_DEC(ucWordFontnumber);
-	DBG_HEX(ucFontstyle);
+	DBG_DEC(ucWordFontNumber);
+	DBG_HEX(usFontStyle);
 	return -1;
 } /* end of iGetFontByNumber */
 
@@ -72,39 +72,39 @@ szGetOurFontname(int iIndex)
  * returns the Word font number, -1 if not found
  */
 int
-iFontname2Fontnumber(const char *szOurFontname, UCHAR ucFontstyle)
+iFontname2Fontnumber(const char *szOurFontname, USHORT usFontStyle)
 {
 	int	iIndex;
 
 	for (iIndex = 0; iIndex < (int)tFontTableRecords; iIndex++) {
-		if (pFontTable[iIndex].ucFontstyle == ucFontstyle &&
+		if (pFontTable[iIndex].usFontStyle == usFontStyle &&
 		    STREQ(pFontTable[iIndex].szOurFontname, szOurFontname)) {
-			return (int)pFontTable[iIndex].ucWordFontnumber;
+			return (int)pFontTable[iIndex].ucWordFontNumber;
 		}
 	}
 	return -1;
-} /* end of iGetFontByName */
+} /* end of iFontname2Fontnumber */
 
 /*
  * See if the fontname from the Word file matches the fontname from the
  * font translation file.
- * If iBytesPerChar is one than szWord is in ISO-8859-x (Word 2/6/7),
- * if iBytesPerChar is two than szWord is in Unicode (Word 8/9/10).
+ * If iBytesPerChar is one than aucWord is in ISO-8859-x (Word 2/6/7),
+ * if iBytesPerChar is two than aucWord is in Unicode (Word 8/9/10).
  */
 static BOOL
-bFontEqual(const UCHAR *szWord, const char *szTable, int iBytesPerChar)
+bFontEqual(const UCHAR *aucWord, const char *szTable, int iBytesPerChar)
 {
 	const UCHAR	*pucTmp;
 	const char	*pcTmp;
 
-	fail(szWord == NULL || szTable == NULL);
+	fail(aucWord == NULL || szTable == NULL);
 	fail(iBytesPerChar != 1 && iBytesPerChar != 2);
 
-	for (pucTmp = szWord, pcTmp = szTable;
+	for (pucTmp = aucWord, pcTmp = szTable;
 	     *pucTmp != 0;
 	     pucTmp += iBytesPerChar, pcTmp++) {
 		if (ulToUpper((ULONG)*pucTmp) !=
-		    ulToUpper((ULONG)*pcTmp)) {
+		    ulToUpper((ULONG)(UCHAR)*pcTmp)) {
 			return FALSE;
 		}
 	}
@@ -112,7 +112,7 @@ bFontEqual(const UCHAR *szWord, const char *szTable, int iBytesPerChar)
 } /* end of bFontEqual */
 
 /*
- *
+ * vFontname2Table - add fontnames to the font table
  */
 static void
 vFontname2Table(const UCHAR *aucFont, const UCHAR *aucAltFont,
@@ -183,11 +183,13 @@ vFontname2Table(const UCHAR *aucFont, const UCHAR *aucAltFont,
 	if (bMatchFound) {
 		switch (iBytesPerChar) {
 		case 1:
-			(void)memcpy(pFontTableRecord->szWordFontname, aucFont,
+			(void)strncpy(pFontTableRecord->szWordFontname,
+				(const char *)aucFont,
 				sizeof(pFontTableRecord->szWordFontname) - 1);
 			break;
 		case 2:
-			(void)unincpy(pFontTableRecord->szWordFontname, aucFont,
+			(void)unincpy(pFontTableRecord->szWordFontname,
+				aucFont,
 				sizeof(pFontTableRecord->szWordFontname) - 1);
 			break;
 		default:
@@ -213,7 +215,6 @@ static void
 vCreateFontTable(void)
 {
 	font_table_type	*pTmp;
-	size_t	tSize;
 	int	iNbr;
 
 	if (tFontTableRecords == 0) {
@@ -222,27 +223,25 @@ vCreateFontTable(void)
 	}
 
 	/* Create the font table */
-	tSize = tFontTableRecords * sizeof(*pFontTable);
-	pFontTable = xmalloc(tSize);
+	pFontTable = xcalloc(tFontTableRecords, sizeof(*pFontTable));
 
 	/* Initialize the font table */
-	(void)memset(pFontTable, 0, tSize);
 	for (iNbr = 0, pTmp = pFontTable;
 	     pTmp < pFontTable + tFontTableRecords;
 	     iNbr++, pTmp++) {
-		pTmp->ucWordFontnumber = (UCHAR)(iNbr / 4);
+		pTmp->ucWordFontNumber = (UCHAR)(iNbr / 4);
 		switch (iNbr % 4) {
 		case 0:
-			pTmp->ucFontstyle = FONT_REGULAR;
+			pTmp->usFontStyle = FONT_REGULAR;
 			break;
 		case 1:
-			pTmp->ucFontstyle = FONT_BOLD;
+			pTmp->usFontStyle = FONT_BOLD;
 			break;
 		case 2:
-			pTmp->ucFontstyle = FONT_ITALIC;
+			pTmp->usFontStyle = FONT_ITALIC;
 			break;
 		case 3:
-			pTmp->ucFontstyle = FONT_BOLD|FONT_ITALIC;
+			pTmp->usFontStyle = FONT_BOLD|FONT_ITALIC;
 			break;
 		default:
 			DBG_DEC(iNbr);
@@ -271,33 +270,35 @@ vMinimizeFontTable(void)
 		return;
 	}
 
+	/* See if we must add a font for our tables */
+	bMustAddTableFont = TRUE;
+
 #if 0
 	DBG_MSG("Before");
 	DBG_DEC(tFontTableRecords);
 	for (pTmp = pFontTable;
 	     pTmp < pFontTable + tFontTableRecords;
 	     pTmp++) {
-		DBG_DEC(pTmp->ucWordFontnumber);
-		DBG_HEX(pTmp->ucFontstyle);
+		DBG_DEC(pTmp->ucWordFontNumber);
+		DBG_HEX(pTmp->usFontStyle);
 		DBG_MSG(pTmp->szWordFontname);
 		DBG_MSG(pTmp->szOurFontname);
 	}
 #endif /* DEBUG */
 
+	/* See which fonts/styles we really need */
+
 	/* Default font/style is by definition in use */
 	pFontTable[0].ucInUse = 1;
 
-	/* See which fonts/styles are really being used */
-	bMustAddTableFont = TRUE;
-
-	/* The fonts/styles that will be used */
+	/* Make InUse 1 for all the fonts/styles that WILL be used */
 	pFont = NULL;
 	while((pFont = pGetNextFontInfoListItem(pFont)) != NULL) {
-		pTmp = pFontTable + 4 * (int)pFont->ucFontnumber;
-		if (bIsBold(pFont->ucFontstyle)) {
+		pTmp = pFontTable + 4 * (int)pFont->ucFontNumber;
+		if (bIsBold(pFont->usFontStyle)) {
 			pTmp++;
 		}
-		if (bIsItalic(pFont->ucFontstyle)) {
+		if (bIsItalic(pFont->usFontStyle)) {
 			pTmp += 2;
 		}
 		if (pTmp >= pFontTable + tFontTableRecords) {
@@ -310,16 +311,16 @@ vMinimizeFontTable(void)
 		pTmp->ucInUse = 1;
 	}
 
-	/* The fonts/styles that might be used */
+	/* Make InUse 1 for all the fonts/styles that MIGHT be used */
 	pStyle = NULL;
 	while((pStyle = pGetNextStyleInfoListItem(pStyle)) != NULL) {
 		vFillFontFromStylesheet(pStyle->usIstdNext, &tFontNext);
 		vCorrectFontValues(&tFontNext);
-		pTmp = pFontTable + 4 * (int)tFontNext.ucFontnumber;
-		if (bIsBold(tFontNext.ucFontstyle)) {
+		pTmp = pFontTable + 4 * (int)tFontNext.ucFontNumber;
+		if (bIsBold(tFontNext.usFontStyle)) {
 			pTmp++;
 		}
-		if (bIsItalic(tFontNext.ucFontstyle)) {
+		if (bIsItalic(tFontNext.usFontStyle)) {
 			pTmp += 2;
 		}
 		if (pTmp >= pFontTable + tFontTableRecords) {
@@ -346,11 +347,15 @@ vMinimizeFontTable(void)
 			*(pTmp - iUnUsed) = *pTmp;
 		}
 	}
+	fail(iUnUsed < 0);
+	fail(tFontTableRecords <= (size_t)iUnUsed);
 	tFontTableRecords -= (size_t)iUnUsed;
+
 	if (bMustAddTableFont) {
 		pTmp = pFontTable + tFontTableRecords;
-		pTmp->ucWordFontnumber = (pTmp - 1)->ucWordFontnumber + 1;
-		pTmp->ucFontstyle = FONT_REGULAR;
+		fail(pTmp <= pFontTable);
+		pTmp->ucWordFontNumber = (pTmp - 1)->ucWordFontNumber + 1;
+		pTmp->usFontStyle = FONT_REGULAR;
 		pTmp->ucInUse = 1;
 		strcpy(pTmp->szWordFontname, "Extra Table Font");
 		strcpy(pTmp->szOurFontname, TABLE_FONT);
@@ -368,8 +373,8 @@ vMinimizeFontTable(void)
 	for (pTmp = pFontTable;
 	     pTmp < pFontTable + tFontTableRecords;
 	     pTmp++) {
-		DBG_DEC(pTmp->ucWordFontnumber);
-		DBG_HEX(pTmp->ucFontstyle);
+		DBG_DEC(pTmp->ucWordFontNumber);
+		DBG_HEX(pTmp->usFontStyle);
 		DBG_MSG(pTmp->szWordFontname);
 		DBG_MSG(pTmp->szOurFontname);
 	}
@@ -430,11 +435,73 @@ bReadFontFile(FILE *pFontTableFile, char *szWordFont,
 	return FALSE;
 } /* end of bReadFontFile */
 
+/*
+ * vCreate0FontTable - create a font table from Word for DOS
+ */
+void
+vCreate0FontTable(void)
+{
+	FILE	*pFontTableFile;
+	font_table_type	*pTmp;
+	UCHAR	*aucFont;
+	int	iBold, iItalic, iSpecial, iEmphasis, iFtc;
+	UCHAR	ucPrq, ucFf, ucFFN;
+	char	szWordFont[FONT_LINE_LENGTH], szOurFont[FONT_LINE_LENGTH];
+
+	tFontTableRecords = 0;
+	pFontTable = xfree(pFontTable);
+
+	pFontTableFile = pOpenFontTableFile();
+	if (pFontTableFile == NULL) {
+		/* No translation table file, no translation table */
+		return;
+	}
+
+	/* Get the maximum number of entries in the font table */
+	tFontTableRecords = 64;
+	tFontTableRecords *= 4;	/* Plain, Bold, Italic and Bold/italic */
+	tFontTableRecords++;	/* One extra for the table-font */
+	vCreateFontTable();
+
+	/* Read the font translation file */
+	iItalic = 0;
+	iBold = 0;
+	iSpecial = 0;
+	while (bReadFontFile(pFontTableFile, szWordFont,
+			&iItalic, &iBold, szOurFont, &iSpecial)) {
+		iEmphasis = 0;
+		if (iBold != 0) {
+			iEmphasis++;
+		}
+		if (iItalic != 0) {
+			iEmphasis += 2;
+		}
+		for (iFtc = 0, pTmp = pFontTable + iEmphasis;
+		     pTmp < pFontTable + tFontTableRecords;
+		     iFtc++, pTmp += 4) {
+			if (iFtc >= 16 && iFtc <= 55) {
+				ucPrq = PITCH_VARIABLE;
+				ucFf = FAMILY_ROMAN;
+				aucFont = (UCHAR *)"Times";
+			} else {
+				ucPrq = PITCH_FIXED;
+				ucFf = FAMILY_MODERN;
+				aucFont = (UCHAR *)"Courier";
+			}
+			ucFFN = (ucFf << 4) | ucPrq;
+			vFontname2Table(aucFont, NULL, 1, iEmphasis,
+					ucFFN, szWordFont, szOurFont, pTmp);
+		}
+	}
+	(void)fclose(pFontTableFile);
+	vMinimizeFontTable();
+} /* end of vCreate0FontTable */
+
 /*
  * vCreate2FontTable - create a font table from WinWord 1/2
  */
 void
-vCreate2FontTable(FILE *pFile, const UCHAR *aucHeader)
+vCreate2FontTable(FILE *pFile, int iWordVersion, const UCHAR *aucHeader)
 {
 	FILE	*pFontTableFile;
 	font_table_type	*pTmp;
@@ -442,12 +509,13 @@ vCreate2FontTable(FILE *pFile, const UCHAR *aucHeader)
 	UCHAR	*aucBuffer;
 	ULONG	ulBeginFontInfo;
 	size_t	tFontInfoLen;
-	int	iPos, iRecLen;
+	int	iPos, iOff, iRecLen;
 	int	iBold, iItalic, iSpecial, iEmphasis;
 	UCHAR	ucFFN;
 	char	szWordFont[FONT_LINE_LENGTH], szOurFont[FONT_LINE_LENGTH];
 
 	fail(pFile == NULL || aucHeader == NULL);
+	fail(iWordVersion != 1 && iWordVersion != 2);
 
 	tFontTableRecords = 0;
 	pFontTable = xfree(pFontTable);
@@ -462,7 +530,6 @@ vCreate2FontTable(FILE *pFile, const UCHAR *aucHeader)
 	DBG_HEX(ulBeginFontInfo);
 	tFontInfoLen = (size_t)usGetWord(0xb6, aucHeader); /* cbSttbfffn */
 	DBG_DEC(tFontInfoLen);
-	fail(tFontInfoLen < 6);
 
 	if (ulBeginFontInfo > (ULONG)LONG_MAX) {
 		/* Don't ask me why this is needed */
@@ -481,20 +548,74 @@ vCreate2FontTable(FILE *pFile, const UCHAR *aucHeader)
 	DBG_DEC(usGetWord(0, aucBuffer));
 
 	/* Compute the maximum number of entries in the font table */
-	tFontTableRecords = 0;
+	if (iWordVersion == 1) {
+		fail(tFontInfoLen < 2);
+		/* WinWord 1 has three implicit fonts */
+		tFontTableRecords = 3;
+		iOff = 2;
+	} else {
+		fail(tFontInfoLen < 6);
+		/* WinWord 2 and up have no implicit fonts */
+		tFontTableRecords = 0;
+		iOff = 3;
+	}
 	iPos = 2;
-	while (iPos + 3 < (int)tFontInfoLen) {
+	while (iPos + iOff < (int)tFontInfoLen) {
 		iRecLen = (int)ucGetByte(iPos, aucBuffer);
 		NO_DBG_DEC(iRecLen);
-		NO_DBG_MSG(aucBuffer + iPos + 3);
+		NO_DBG_MSG(aucBuffer + iPos + iOff);
 		iPos += iRecLen + 1;
 		tFontTableRecords++;
 	}
-	tFontTableRecords *= 4;	/* Plain, Bold, Italic and Bold/italic */
+	tFontTableRecords *= 4;	/* Plain, Bold, Italic and Bold/Italic */
 	tFontTableRecords++;	/* One extra for the table-font */
 	vCreateFontTable();
 
+	/* Add the tree implicit fonts (in four variations) */
+	if (iWordVersion == 1) {
+		fail(tFontTableRecords < 13);
+		vFontname2Table((UCHAR *)"Tms Rmn", NULL, 1, 0,
+			(UCHAR)((FAMILY_ROMAN << 4) | PITCH_VARIABLE),
+			"*", "Times-Roman", pFontTable + 0);
+		vFontname2Table((UCHAR *)"Tms Rmn", NULL, 1, 1,
+			(UCHAR)((FAMILY_ROMAN << 4) | PITCH_VARIABLE),
+			"*", "Times-Bold", pFontTable + 1);
+		vFontname2Table((UCHAR *)"Tms Rmn", NULL, 1, 2,
+			(UCHAR)((FAMILY_ROMAN << 4) | PITCH_VARIABLE),
+			"*", "Times-Italic", pFontTable + 2);
+		vFontname2Table((UCHAR *)"Tms Rmn", NULL, 1, 3,
+			(UCHAR)((FAMILY_ROMAN << 4) | PITCH_VARIABLE),
+			"*", "Times-BoldItalic", pFontTable + 3);
+		vFontname2Table((UCHAR *)"Symbol", NULL, 1, 0,
+			(UCHAR)((FAMILY_ROMAN << 4) | PITCH_VARIABLE),
+			"*", "Times-Roman", pFontTable + 4);
+		vFontname2Table((UCHAR *)"Symbol", NULL, 1, 1,
+			(UCHAR)((FAMILY_ROMAN << 4) | PITCH_VARIABLE),
+			"*", "Times-Bold", pFontTable + 5);
+		vFontname2Table((UCHAR *)"Symbol", NULL, 1, 2,
+			(UCHAR)((FAMILY_ROMAN << 4) | PITCH_VARIABLE),
+			"*", "Times-Italic", pFontTable + 6);
+		vFontname2Table((UCHAR *)"Symbol", NULL, 1, 3,
+			(UCHAR)((FAMILY_ROMAN << 4) | PITCH_VARIABLE),
+			"*", "Times-BoldItalic", pFontTable + 7);
+		vFontname2Table((UCHAR *)"Helv", NULL, 1, 0,
+			(UCHAR)((FAMILY_SWISS << 4) | PITCH_VARIABLE),
+			"*", "Helvetica", pFontTable + 8);
+		vFontname2Table((UCHAR *)"Helv", NULL, 1, 1,
+			(UCHAR)((FAMILY_SWISS << 4) | PITCH_VARIABLE),
+			"*", "Helvetica-Bold", pFontTable + 9);
+		vFontname2Table((UCHAR *)"Helv", NULL, 1, 2,
+			(UCHAR)((FAMILY_SWISS << 4) | PITCH_VARIABLE),
+			"*", "Helvetica-Oblique", pFontTable + 10);
+		vFontname2Table((UCHAR *)"Helv", NULL, 1, 3,
+			(UCHAR)((FAMILY_SWISS << 4) | PITCH_VARIABLE),
+			"*", "Helvetica-BoldOblique", pFontTable + 11);
+	}
+
 	/* Read the font translation file */
+	iItalic = 0;
+	iBold = 0;
+	iSpecial = 0;
 	while (bReadFontFile(pFontTableFile, szWordFont,
 			&iItalic, &iBold, szOurFont, &iSpecial)) {
 		iEmphasis = 0;
@@ -506,10 +627,10 @@ vCreate2FontTable(FILE *pFile, const UCHAR *aucHeader)
 		}
 		pTmp = pFontTable + iEmphasis;
 		iPos = 2;
-		while (iPos + 3 < (int)tFontInfoLen) {
+		while (iPos + iOff < (int)tFontInfoLen) {
 			iRecLen = (int)ucGetByte(iPos, aucBuffer);
 			ucFFN = ucGetByte(iPos + 1, aucBuffer);
-			aucFont = aucBuffer + iPos + 3;
+			aucFont = aucBuffer + iPos + iOff;
 			vFontname2Table(aucFont, NULL, 1, iEmphasis,
 					ucFFN, szWordFont, szOurFont, pTmp);
 			pTmp += 4;
@@ -587,6 +708,9 @@ vCreate6FontTable(FILE *pFile, ULONG ulStartBlock,
 	vCreateFontTable();
 
 	/* Read the font translation file */
+	iItalic = 0;
+	iBold = 0;
+	iSpecial = 0;
 	while (bReadFontFile(pFontTableFile, szWordFont,
 			&iItalic, &iBold, szOurFont, &iSpecial)) {
 		iEmphasis = 0;
@@ -707,6 +831,9 @@ vCreate8FontTable(FILE *pFile, const pps_info_type *pPPS,
 	vCreateFontTable();
 
 	/* Read the font translation file */
+	iItalic = 0;
+	iBold = 0;
+	iSpecial = 0;
 	while (bReadFontFile(pFontTableFile, szWordFont,
 			&iItalic, &iBold, szOurFont, &iSpecial)) {
 		iEmphasis = 0;
@@ -761,17 +888,25 @@ vDestroyFontTable(void)
 const font_table_type *
 pGetNextFontTableRecord(const font_table_type *pRecordCurr)
 {
-	int	iIndexCurr;
+	size_t	tIndexCurr;
 
 	if (pRecordCurr == NULL) {
-		/* No current record, so start with the first */
+		/* No current record, so start with the first one */
 		return &pFontTable[0];
 	}
 
-	iIndexCurr = pRecordCurr - pFontTable;
-	if (iIndexCurr + 1 < (int)tFontTableRecords) {
+	if (pRecordCurr < pFontTable ||
+	    pRecordCurr >= pFontTable + tFontTableRecords) {
+		/* Not a pointer in the array */
+		DBG_HEX(pRecordCurr);
+		DBG_HEX(pFontTable);
+		return NULL;
+	}
+
+	tIndexCurr = pRecordCurr - pFontTable;
+	if (tIndexCurr + 1 < tFontTableRecords) {
 		/* There is a next record, so return it */
-		return &pFontTable[iIndexCurr + 1];
+		return &pFontTable[tIndexCurr + 1];
 	}
 	/* There is no next record */
 	return NULL;
