@@ -27,10 +27,34 @@ FILE *
 pOpenFontTableFile(void)
 {
 	FILE		*pFile;
-	const char	*szHome;
-	const char	*szGlobalFile;
+	const char	*szHome, *szAntiword, *szGlobalFile;
+	char		szEnvironmentFile[PATH_MAX+1];
 	char		szLocalFile[PATH_MAX+1];
 
+	szEnvironmentFile[0] = '\0';
+	szLocalFile[0] = '\0';
+
+	/* Try the environment version of the fontnames file */
+	szAntiword = szGetAntiwordDirectory();
+	if (szAntiword != NULL && szAntiword[0] != '\0') {
+		if (strlen(szAntiword) +
+		    sizeof(FILE_SEPARATOR FONTNAMES_FILE) >=
+		    sizeof(szEnvironmentFile)) {
+			werr(0,
+			"The name of your ANTIWORDHOME directory is too long");
+			return NULL;
+		}
+		sprintf(szEnvironmentFile, "%s%s",
+			szAntiword,
+			FILE_SEPARATOR FONTNAMES_FILE);
+		DBG_MSG(szEnvironmentFile);
+
+		pFile = fopen(szEnvironmentFile, "r");
+		if (pFile != NULL) {
+			return pFile;
+		}
+	}
+
 	/* Try the local version of the fontnames file */
 	szHome = szGetHomeDirectory();
 	if (strlen(szHome) +
@@ -59,10 +83,18 @@ pOpenFontTableFile(void)
 		return pFile;
 	}
 
-	werr(0, "I can not open your fontnames file.\n"
-		"Neither '%s' nor\n"
-		"'%s' can be opened for reading.",
-		szLocalFile, szGlobalFile);
+	if (szEnvironmentFile[0] != '\0') {
+		werr(0, "I can not open your fontnames file.\n"
+			"Neither '%s' nor\n"
+			"'%s' nor\n"
+			"'%s' can be opened for reading.",
+			szEnvironmentFile, szLocalFile, szGlobalFile);
+	} else {
+		werr(0, "I can not open your fontnames file.\n"
+			"Neither '%s' nor\n"
+			"'%s' can be opened for reading.",
+			szLocalFile, szGlobalFile);
+	}
 	return NULL;
 } /* end of pOpenFontTableFile */
 
@@ -79,12 +111,12 @@ vCloseFont(void)
 } /* end of vCloseFont */
 
 /*
- * tOpenFont - make the given font the current font
+ * tOpenFont - make the specified font the current font
  *
  * Returns the font reference number
  */
 draw_fontref
-tOpenFont(UCHAR ucWordFontNumber, UCHAR ucFontstyle, short sWordFontsize)
+tOpenFont(UCHAR ucWordFontNumber, USHORT usFontStyle, USHORT usWordFontSize)
 {
 	options_type	tOptions;
 	const char	*szOurFontname;
@@ -92,23 +124,24 @@ tOpenFont(UCHAR ucWordFontNumber, UCHAR ucFontstyle, short sWordFontsize)
 
 	NO_DBG_MSG("tOpenFont");
 	NO_DBG_DEC(ucWordFontNumber);
-	NO_DBG_HEX(ucFontstyle);
-	NO_DBG_DEC(sWordFontsize);
+	NO_DBG_HEX(usFontStyle);
+	NO_DBG_DEC(usWordFontSize);
 
 	/* Keep the relevant bits */
-	ucFontstyle &= FONT_BOLD|FONT_ITALIC;
-	NO_DBG_HEX(ucFontstyle);
+	usFontStyle &= FONT_BOLD|FONT_ITALIC;
+	NO_DBG_HEX(usFontStyle);
 
 	vGetOptions(&tOptions);
-	bUsePlainText = !tOptions.bUseOutlineFonts;
+	bUsePlainText = tOptions.eConversionType != conversion_draw &&
+			tOptions.eConversionType != conversion_ps;
 	eEncoding = tOptions.eEncoding;
 
 	if (bUsePlainText) {
-		/* No outline fonts, no postscript just plain text */
+		/* Plain text, no fonts */
 		return (draw_fontref)0;
 	}
 
-	iFontnumber = iGetFontByNumber(ucWordFontNumber, ucFontstyle);
+	iFontnumber = iGetFontByNumber(ucWordFontNumber, usFontStyle);
 	szOurFontname = szGetOurFontname(iFontnumber);
 	if (szOurFontname == NULL || szOurFontname[0] == '\0') {
 		DBG_DEC(iFontnumber);
@@ -131,7 +164,7 @@ tOpenFont(UCHAR ucWordFontNumber, UCHAR ucFontstyle, short sWordFontsize)
  * Returns the font reference number
  */
 draw_fontref
-tOpenTableFont(short sWordFontsize)
+tOpenTableFont(USHORT usWordFontSize)
 {
 	options_type	tOptions;
 	int	iWordFontnumber;
@@ -139,11 +172,12 @@ tOpenTableFont(short sWordFontsize)
 	NO_DBG_MSG("tOpenTableFont");
 
 	vGetOptions(&tOptions);
-	bUsePlainText = !tOptions.bUseOutlineFonts;
+	bUsePlainText = tOptions.eConversionType != conversion_draw &&
+			tOptions.eConversionType != conversion_ps;
 	eEncoding = tOptions.eEncoding;
 
 	if (bUsePlainText) {
-		/* No outline fonts, no postscript, just plain text */
+		/* Plain text, no fonts */
 		return (draw_fontref)0;
 	}
 
@@ -153,7 +187,7 @@ tOpenTableFont(short sWordFontsize)
 		return (draw_fontref)0;
 	}
 
-	return tOpenFont((UCHAR)iWordFontnumber, FONT_REGULAR, sWordFontsize);
+	return tOpenFont((UCHAR)iWordFontnumber, FONT_REGULAR, usWordFontSize);
 } /* end of tOpenTableFont */
 
 /*
@@ -169,24 +203,25 @@ szGetFontname(draw_fontref tFontRef)
 /*
  * lComputeStringWidth - compute the string width
  *
- * Note: the fontsize is given in half-points!
- *       the stringlength is given in bytes, not characters!
+ * Note: the fontsize is specified in half-points!
+ *       the stringlength is specified in bytes, not characters!
  *
  * Returns the string width in millipoints
  */
 long
-lComputeStringWidth(char *szString, int iStringLength,
-		draw_fontref tFontRef, short sFontsize)
+lComputeStringWidth(const char *szString, size_t tStringLength,
+		draw_fontref tFontRef, USHORT usFontSize)
 {
 	USHORT	*ausCharWidths;
 	UCHAR	*pucChar;
 	long	lRelWidth;
-	int	iIndex, iFontRef;
+	size_t	tIndex;
+	int	iFontRef;
 
-	fail(szString == NULL || iStringLength < 0);
-	fail(sFontsize < MIN_FONT_SIZE || sFontsize > MAX_FONT_SIZE);
+	fail(szString == NULL);
+	fail(usFontSize < MIN_FONT_SIZE || usFontSize > MAX_FONT_SIZE);
 
-	if (szString[0] == '\0' || iStringLength <= 0) {
+	if (szString[0] == '\0' || tStringLength == 0) {
 		/* Empty string */
 		return 0;
 	}
@@ -194,12 +229,12 @@ lComputeStringWidth(char *szString, int iStringLength,
 	if (eEncoding == encoding_utf8) {
 		fail(!bUsePlainText);
 		return lChar2MilliPoints(
-				utf8_strwidth(szString, iStringLength));
+			utf8_strwidth(szString, tStringLength));
 	}
 
 	if (bUsePlainText) {
 		/* No current font, use "systemfont" */
-		return lChar2MilliPoints(iStringLength);
+		return lChar2MilliPoints(tStringLength);
 	}
 
 	DBG_DEC_C(eEncoding != encoding_iso_8859_1 &&
@@ -215,12 +250,48 @@ lComputeStringWidth(char *szString, int iStringLength,
 		ausCharWidths = ausCharacterWidths1[iFontRef];
 	}
 	lRelWidth = 0;
-	for (iIndex = 0, pucChar = (UCHAR *)szString;
-	     iIndex < iStringLength;
-	     iIndex++, pucChar++) {
+	for (tIndex = 0, pucChar = (UCHAR *)szString;
+	     tIndex < tStringLength;
+	     tIndex++, pucChar++) {
 		lRelWidth += (long)ausCharWidths[(int)*pucChar];
 	}
 
 	/* Compute the absolute string width */
-	return (lRelWidth * sFontsize + 1) / 2;
+	return (lRelWidth * usFontSize + 1) / 2;
 } /* end of lComputeStringWidth */
+
+/*
+ * tCountColumns - count the number of columns in a string
+ *
+ * Note: the length is specified in bytes!
+ *       A UTF-8 a character can be 0, 1 or 2 columns wide.
+ *
+ * Returns the number of columns
+ */
+size_t
+tCountColumns(const char *szString, size_t tLength)
+{
+	fail(szString == NULL);
+
+	if (eEncoding != encoding_utf8) {
+		/* One byte, one character, one column */
+		return tLength;
+	}
+	return (size_t)utf8_strwidth(szString, tLength);
+} /* end of tCountColumns */
+
+/*
+ * tGetCharacterLength - the length of the specified character in bytes
+ *
+ * Returns the length in bytes
+ */
+size_t
+tGetCharacterLength(const char *szString)
+{
+	fail(szString == NULL);
+
+	if (eEncoding != encoding_utf8) {
+		return 1;
+	}
+	return (size_t)utf8_chrlength(szString);
+} /* end of tGetCharacterLength */
