@@ -1,12 +1,12 @@
 /*
  * prop8.c
- * Copyright (C) 1998-2002 A.J. van Os; Released under GPL
+ * Copyright (C) 1998-2003 A.J. van Os; Released under GPL
  *
  * Description:
  * Read the property information from a MS Word 8, 9 or 10 file
  *
- * Word 8 is better known as Word 97
- * Word 9 is better known as Word 2000
+ * Word  8 is better known as Word 97 or as Word 98 for Mac
+ * Word  9 is better known as Word 2000 or as Word 2001 for Mac
  * Word 10 is better known as Word 2002 or as Word XP
  */
 
@@ -14,6 +14,8 @@
 #include <string.h>
 #include "antiword.h"
 
+#define DEFAULT_LISTCHAR	0x002e	/* A full stop */
+
 
 /*
  * iGet8InfoLength - the length of the information for Word 8/9/10 files
@@ -59,7 +61,8 @@ static void
 vGet8SectionInfo(const UCHAR *aucGrpprl, size_t tBytes,
 		section_block_type *pSection)
 {
-	int	iFodoOff, iInfoLen, iSize, iIndex, iTmp;
+	UINT	uiIndex;
+	int	iFodoOff, iInfoLen, iSize, iTmp;
 	USHORT	usCcol;
 	UCHAR	ucTmp;
 
@@ -81,21 +84,21 @@ vGet8SectionInfo(const UCHAR *aucGrpprl, size_t tBytes,
 		case 0xd202:	/* olstAnm */
 			iSize = (int)ucGetByte(iFodoOff + 2, aucGrpprl);
 			DBG_DEC_C(iSize != 212, iSize);
-			for (iIndex = 0, iTmp = iFodoOff + 3;
-			     iIndex < 9 && iTmp < iFodoOff + 3 + iSize - 15;
-			     iIndex++, iTmp += 16) {
-				pSection->aucNFC[iIndex] =
+			for (uiIndex = 0, iTmp = iFodoOff + 3;
+			     uiIndex < 9 && iTmp < iFodoOff + 3 + iSize - 15;
+			     uiIndex++, iTmp += 16) {
+				pSection->aucNFC[uiIndex] =
 						ucGetByte(iTmp, aucGrpprl);
-				DBG_DEC(pSection->aucNFC[iIndex]);
+				DBG_DEC(pSection->aucNFC[uiIndex]);
 				ucTmp = ucGetByte(iTmp + 3, aucGrpprl);
 				DBG_HEX(ucTmp);
 				if ((ucTmp & BIT(2)) != 0) {
 					pSection->usNeedPrevLvl |=
-							(USHORT)BIT(iIndex);
+							(USHORT)BIT(uiIndex);
 				}
 				if ((ucTmp & BIT(3)) != 0) {
 					pSection->usHangingIndent |=
-							(USHORT)BIT(iIndex);
+							(USHORT)BIT(uiIndex);
 				}
 			}
 			DBG_HEX(pSection->usNeedPrevLvl);
@@ -128,7 +131,7 @@ vGet8SepInfo(FILE *pFile, const pps_info_type *pPPS,
 	ULONG	ulBeginSectInfo;
 	ULONG	ulTableSize, ulTableStartBlock;
 	size_t	tSectInfoLen, tBlockDepotLen;
-	size_t	tBlockSize, tOffset, tSize, tLen, tBytes;
+	size_t	tBlockSize, tOffset, tLen, tBytes;
 	int	iIndex;
 	USHORT	usDocStatus;
 	UCHAR	aucTmp[2];
@@ -161,12 +164,12 @@ vGet8SepInfo(FILE *pFile, const pps_info_type *pPPS,
 	}
 	DBG_HEX(ulTableSize);
 	if (ulTableSize < MIN_SIZE_FOR_BBD_USE) {
-	  	/* Use the Small Block Depot */
+		/* Use the Small Block Depot */
 		aulBlockDepot = aulSBD;
 		tBlockDepotLen = tSBDLen;
 		tBlockSize = SMALL_BLOCK_SIZE;
 	} else {
-	  	/* Use the Big Block Depot */
+		/* Use the Big Block Depot */
 		aulBlockDepot = aulBBD;
 		tBlockDepotLen = tBBDLen;
 		tBlockSize = BIG_BLOCK_SIZE;
@@ -182,16 +185,15 @@ vGet8SepInfo(FILE *pFile, const pps_info_type *pPPS,
 
 	/* Read the Section Descriptors */
 	tLen = (tSectInfoLen - 4) / 16;
-	tSize = tLen * sizeof(ULONG);
 	/* Save the section offsets */
-	aulTextOffset = xmalloc(tSize);
+	aulTextOffset = xcalloc(tLen, sizeof(ULONG));
 	for (iIndex = 0, tOffset = 0;
 	     iIndex < (int)tLen;
 	     iIndex++, tOffset += 4) {
 		aulTextOffset[iIndex] = ulGetLong(tOffset, aucBuffer);
 	}
 	/* Save the Sepx offsets */
-	aulSectPage = xmalloc(tSize);
+	aulSectPage = xcalloc(tLen, sizeof(ULONG));
 	for (iIndex = 0, tOffset = (tLen + 1) * 4;
 	     iIndex < (int)tLen;
 	     iIndex++, tOffset += 12) {
@@ -243,6 +245,7 @@ eGet8RowInfo(int iFodo,
 	int	iFodoOff, iInfoLen;
 	int	iIndex, iSize, iCol;
 	int	iPosCurr, iPosPrev;
+	USHORT	usTmp;
 	BOOL	bFound2416_0, bFound2416_1, bFound2417_0, bFound2417_1;
 	BOOL	bFoundd608;
 
@@ -271,6 +274,46 @@ eGet8RowInfo(int iFodo,
 				bFound2417_0 = TRUE;
 			}
 			break;
+		case 0x6424:	/* brcTop */
+			usTmp = usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
+			usTmp &= 0xff00;
+			NO_DBG_DEC(usTmp >> 8);
+			if (usTmp == 0) {
+				pRow->ucBorderInfo &= ~TABLE_BORDER_TOP;
+			} else {
+				pRow->ucBorderInfo |= TABLE_BORDER_TOP;
+			}
+			break;
+		case 0x6425:	/* brcLeft */
+			usTmp = usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
+			usTmp &= 0xff00;
+			NO_DBG_DEC(usTmp >> 8);
+			if (usTmp == 0) {
+				pRow->ucBorderInfo &= ~TABLE_BORDER_LEFT;
+			} else {
+				pRow->ucBorderInfo |= TABLE_BORDER_LEFT;
+			}
+			break;
+		case 0x6426:	/* brcBottom */
+			usTmp = usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
+			usTmp &= 0xff00;
+			NO_DBG_DEC(usTmp >> 8);
+			if (usTmp == 0) {
+				pRow->ucBorderInfo &= ~TABLE_BORDER_BOTTOM;
+			} else {
+				pRow->ucBorderInfo |= TABLE_BORDER_BOTTOM;
+			}
+			break;
+		case 0x6427:	/* brcRight */
+			usTmp = usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
+			usTmp &= 0xff00;
+			NO_DBG_DEC(usTmp >> 8);
+			if (usTmp == 0) {
+				pRow->ucBorderInfo &= ~TABLE_BORDER_RIGHT;
+			} else {
+				pRow->ucBorderInfo |= TABLE_BORDER_RIGHT;
+			}
+			break;
 		case 0xd608:	/* cDefTable */
 			iSize = (int)usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
 			if (iSize < 6 || iBytes < iFodoOff + 8) {
@@ -341,16 +384,20 @@ void
 vGet8StyleInfo(int iFodo,
 	const UCHAR *aucGrpprl, int iBytes, style_block_type *pStyle)
 {
+	list_block_type	tList6;
+	const list_block_type	*pList;
 	int	iFodoOff, iInfoLen;
 	int	iTmp, iDel, iAdd, iBefore;
-	USHORT	usOpCode, usIlfo, usTmp;
+	USHORT	usOpCode, usTmp;
 	short	sTmp;
 
 	fail(iFodo < 0 || aucGrpprl == NULL || pStyle == NULL);
 
-	NO_DBG_DEC(pStyle->usIstd);
+	NO_DBG_DEC_C(pStyle->usListIndex != 0, pStyle->usIstd);
+	NO_DBG_DEC_C(pStyle->usListIndex != 0, pStyle->usListIndex);
+
+	(void)memset(&tList6, 0, sizeof(tList6));
 
-	usIlfo = 0;
 	iFodoOff = 0;
 	while (iBytes >= iFodoOff + 2) {
 		iInfoLen = 0;
@@ -361,19 +408,19 @@ vGet8StyleInfo(int iFodo,
 					iFodo + iFodoOff + 2, aucGrpprl);
 			break;
 		case 0x260a:	/* ilvl */
-			pStyle->bInList = TRUE;
 			pStyle->ucListLevel =
 				ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			NO_DBG_DEC(pStyle->ucListLevel);
+			pStyle->ucNumLevel = pStyle->ucListLevel;
 			break;
 		case 0x4600:	/* istd */
 			usTmp = usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
 			NO_DBG_DEC(usTmp);
 			break;
 		case 0x460b:	/* ilfo */
-			pStyle->bInList = TRUE;
-			usIlfo = usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
-			pStyle->bNumPause = usIlfo == 0;
+			pStyle->usListIndex =
+				usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
+			NO_DBG_DEC(pStyle->usListIndex);
 			break;
 		case 0x4610: /* Nest dxaLeft */
 			sTmp = (short)usGetWord(
@@ -416,6 +463,11 @@ vGet8StyleInfo(int iFodo,
 					iFodo + iFodoOff + 2, aucGrpprl);
 			NO_DBG_DEC(pStyle->sLeftIndent);
 			break;
+		case 0x8411:	/* dxaLeft1 */
+			pStyle->sLeftIndent1 = (short)usGetWord(
+					iFodo + iFodoOff + 2, aucGrpprl);
+			NO_DBG_DEC(pStyle->sLeftIndent1);
+			break;
 		case 0xa413:	/* dyaBefore */
 			pStyle->usBeforeIndent = usGetWord(
 					iFodo + iFodoOff + 2, aucGrpprl);
@@ -427,34 +479,28 @@ vGet8StyleInfo(int iFodo,
 			NO_DBG_DEC(pStyle->usAfterIndent);
 			break;
 		case 0xc63e:	/* anld */
-#if 0
-			if (usIlfo != 2047) {
-				break;
-			}
-#endif
 			iTmp = (int)ucGetByte(
 					iFodo + iFodoOff + 2, aucGrpprl);
 			DBG_DEC_C(iTmp < 84, iTmp);
-			pStyle->bInList = TRUE;
 			if (iTmp >= 1) {
-				pStyle->ucNFC = ucGetByte(
+				tList6.ucNFC = ucGetByte(
 					iFodo + iFodoOff + 3, aucGrpprl);
 			}
-			if (pStyle->ucNFC != LIST_BULLETS && iTmp >= 2) {
+			if (tList6.ucNFC != LIST_BULLETS && iTmp >= 2) {
 				iBefore = (int)ucGetByte(
 					iFodo + iFodoOff + 4, aucGrpprl);
 			} else {
 				iBefore = 0;
 			}
 			if (iTmp >= 12) {
-				pStyle->usStartAt = usGetWord(
+				tList6.ulStartAt = (ULONG)usGetWord(
 					iFodo + iFodoOff + 13, aucGrpprl);
 			}
-			if (iTmp >= iBefore + 21) {
-				pStyle->ucListCharacter = ucGetByte(
+			if (iTmp >= iBefore + 22) {
+				tList6.usListChar = usGetWord(
 					iFodo + iFodoOff + iBefore + 23,
 					aucGrpprl);
-				NO_DBG_CHR(pStyle->ucListCharacter);
+				DBG_HEX(tList6.usListChar);
 			}
 			break;
 		default:
@@ -468,8 +514,281 @@ vGet8StyleInfo(int iFodo,
 		}
 		iFodoOff += iInfoLen;
 	}
+
+	if (pStyle->usListIndex == 2047) {
+		/* Old style list */
+		pStyle->usStartAt = (USHORT)tList6.ulStartAt;
+		pStyle->usListChar = tList6.usListChar;
+		pStyle->ucNFC = tList6.ucNFC;
+	} else {
+		/* New style list */
+		pList = pGetListInfo(pStyle->usListIndex, pStyle->ucListLevel);
+		if (pList != NULL) {
+			pStyle->bNoRestart = pList->bNoRestart;
+			fail(pList->ulStartAt > (ULONG)USHRT_MAX);
+			pStyle->usStartAt = (USHORT)pList->ulStartAt;
+			pStyle->usListChar = pList->usListChar;
+			pStyle->ucNFC = pList->ucNFC;
+			if (pStyle->sLeftIndent <= 0) {
+				pStyle->sLeftIndent = pList->sLeftIndent;
+			}
+		}
+	}
 } /* end of vGet8StyleInfo */
 
+/*
+ * Get the left indentation value from the style information block
+ *
+ * Returns the value when found, otherwise 0
+ */
+static short
+sGetLeftIndent(const UCHAR *aucGrpprl, size_t tBytes)
+{
+	int	iOffset, iInfoLen;
+	USHORT	usOpCode, usTmp;
+
+	fail(aucGrpprl == NULL);
+
+	iOffset = 0;
+	while (tBytes >= (size_t)iOffset + 4) {
+		usOpCode = usGetWord(iOffset, aucGrpprl);
+		if (usOpCode == 0x840f) {	/* dxaLeft */
+			usTmp = usGetWord(iOffset + 2, aucGrpprl);
+			if (usTmp <= 0x7fff) {
+				NO_DBG_DEC(usTmp);
+				return (short)usTmp;
+			}
+		}
+		iInfoLen = iGet8InfoLength(iOffset, aucGrpprl);
+		fail(iInfoLen <= 0);
+		iOffset += iInfoLen;
+	}
+	return 0;
+} /* end of sGetLeftIndent */
+
+/*
+ * Build the list with List Information for Word 8/9/10 files
+ */
+void
+vGet8LstInfo(FILE *pFile, const pps_info_type *pPPS,
+	const ULONG *aulBBD, size_t tBBDLen,
+	const ULONG *aulSBD, size_t tSBDLen,
+	const UCHAR *aucHeader)
+{
+	list_block_type	tList;
+	const ULONG	*aulBlockDepot;
+	UCHAR	*aucLfoInfo, *aucLstfInfo, *aucPapx, *aucXString;
+	ULONG	ulTableStartBlock, ulTableSize;
+	ULONG	ulBeginLfoInfo, ulBeginLstfInfo, ulBeginLvlfInfo;
+	ULONG	ulListID, ulStart;
+	size_t	tBlockDepotLen, tBlockSize;
+	size_t	tLfoInfoLen, tLstfInfoLen, tPapxLen, tXstLen, tOff;
+	size_t	tLstfRecords, tStart, tIndex;
+	int	iNums;
+	USHORT	usDocStatus, usIstd;
+	UCHAR	ucTmp, ucListLevel, ucMaxLevel, ucChpxLen;
+	UCHAR	aucLvlfInfo[28], aucXst[2];
+
+	fail(pFile == NULL || pPPS == NULL || aucHeader == NULL);
+	fail(aulBBD == NULL || aulSBD == NULL);
+
+	/* Use 0Table or 1Table? */
+	usDocStatus = usGetWord(0x0a, aucHeader);
+	if (usDocStatus & BIT(9)) {
+		ulTableStartBlock = pPPS->t1Table.ulSB;
+		ulTableSize = pPPS->t1Table.ulSize;
+	} else {
+		ulTableStartBlock = pPPS->t0Table.ulSB;
+		ulTableSize = pPPS->t0Table.ulSize;
+	}
+	DBG_DEC(ulTableStartBlock);
+	if (ulTableStartBlock == 0) {
+		DBG_MSG("No list information");
+		return;
+	}
+	DBG_HEX(ulTableSize);
+	if (ulTableSize < MIN_SIZE_FOR_BBD_USE) {
+		/* Use the Small Block Depot */
+		aulBlockDepot = aulSBD;
+		tBlockDepotLen = tSBDLen;
+		tBlockSize = SMALL_BLOCK_SIZE;
+	} else {
+		/* Use the Big Block Depot */
+		aulBlockDepot = aulBBD;
+		tBlockDepotLen = tBBDLen;
+		tBlockSize = BIG_BLOCK_SIZE;
+	}
+
+	/* LFO (List Format Override) */
+	ulBeginLfoInfo = ulGetLong(0x2ea, aucHeader); /* fcPlfLfo */
+	DBG_HEX(ulBeginLfoInfo);
+	tLfoInfoLen = (size_t)ulGetLong(0x2ee, aucHeader); /* lcbPlfLfo */
+	DBG_DEC(tLfoInfoLen);
+	if (tLfoInfoLen == 0) {
+		DBG_MSG("No lists in this document");
+		return;
+	}
+
+	aucLfoInfo = xmalloc(tLfoInfoLen);
+	if (!bReadBuffer(pFile, ulTableStartBlock,
+			aulBlockDepot, tBlockDepotLen, tBlockSize,
+			aucLfoInfo, ulBeginLfoInfo, tLfoInfoLen)) {
+		aucLfoInfo = xfree(aucLfoInfo);
+		return;
+	}
+	NO_DBG_PRINT_BLOCK(aucLfoInfo, tLfoInfoLen);
+	vBuildLfoList(aucLfoInfo, tLfoInfoLen);
+	aucLfoInfo = xfree(aucLfoInfo);
+
+	/* LSTF (LiST data on File) */
+	ulBeginLstfInfo = ulGetLong(0x2e2, aucHeader); /* fcPlcfLst */
+	DBG_HEX(ulBeginLstfInfo);
+	tLstfInfoLen = (size_t)ulGetLong(0x2e6, aucHeader); /* lcbPlcfLst */
+	DBG_DEC(tLstfInfoLen);
+	if (tLstfInfoLen == 0) {
+		DBG_MSG("No list data on file");
+		return;
+	}
+
+	aucLstfInfo = xmalloc(tLstfInfoLen);
+	if (!bReadBuffer(pFile, ulTableStartBlock,
+			aulBlockDepot, tBlockDepotLen, tBlockSize,
+			aucLstfInfo, ulBeginLstfInfo, tLstfInfoLen)) {
+		aucLstfInfo = xfree(aucLstfInfo);
+		return;
+	}
+	NO_DBG_PRINT_BLOCK(aucLstfInfo, tLstfInfoLen);
+
+	tLstfRecords = (size_t)usGetWord(0, aucLstfInfo);
+	if (2 + tLstfRecords * 28 < tLstfInfoLen) {
+		DBG_DEC(2 + tLstfRecords * 28);
+		DBG_DEC(tLstfInfoLen);
+		aucLstfInfo = xfree(aucLstfInfo);
+		return;
+	}
+
+	/* LVLF (List leVeL on File) */
+	ulBeginLvlfInfo = ulBeginLstfInfo + tLstfInfoLen;
+	DBG_HEX(ulBeginLvlfInfo);
+
+	aucXString = NULL;
+	ulStart = ulBeginLvlfInfo;
+
+	for (tIndex = 0, tStart = 2;
+	     tIndex < tLstfRecords;
+	     tIndex++, tStart += 28) {
+		ulListID = ulGetLong(tStart, aucLstfInfo);
+		NO_DBG_HEX(ulListID);
+		ucTmp = ucGetByte(tStart + 26, aucLstfInfo);
+		ucMaxLevel = odd(ucTmp) ? 1 : 9;
+		for (ucListLevel = 0; ucListLevel < ucMaxLevel; ucListLevel++) {
+			fail(aucXString != NULL);
+			usIstd = usGetWord(
+					tStart + 8 + 2 * (size_t)ucListLevel,
+					aucLstfInfo);
+			DBG_DEC_C(usIstd != STI_NIL, usIstd);
+			NO_DBG_HEX(ulStart);
+			(void)memset(&tList, 0, sizeof(tList));
+			/* Read the lvlf (List leVeL on File) */
+			if (!bReadBuffer(pFile, ulTableStartBlock,
+					aulBlockDepot, tBlockDepotLen,
+					tBlockSize, aucLvlfInfo,
+					ulStart, sizeof(aucLvlfInfo))) {
+				aucLstfInfo = xfree(aucLstfInfo);
+				return;
+			}
+			NO_DBG_PRINT_BLOCK(aucLvlfInfo, sizeof(aucLvlfInfo));
+			if (bAllZero(aucLvlfInfo, sizeof(aucLvlfInfo))) {
+				tList.ulStartAt = 1;
+				tList.ucNFC = 0x00;
+				tList.bNoRestart = FALSE;
+			} else {
+				tList.ulStartAt = ulGetLong(0, aucLvlfInfo);
+				tList.ucNFC = ucGetByte(4, aucLvlfInfo);
+				ucTmp = ucGetByte(5, aucLvlfInfo);
+				tList.bNoRestart = (ucTmp & BIT(3)) != 0;
+			}
+			ulStart += sizeof(aucLvlfInfo);
+			tPapxLen = (size_t)ucGetByte(25, aucLvlfInfo);
+			if (tPapxLen != 0) {
+				aucPapx = xmalloc(tPapxLen);
+				/* Read the Papx */
+				if (!bReadBuffer(pFile, ulTableStartBlock,
+						aulBlockDepot, tBlockDepotLen,
+						tBlockSize, aucPapx,
+						ulStart, tPapxLen)) {
+					aucPapx = xfree(aucPapx);
+					aucLstfInfo = xfree(aucLstfInfo);
+					return;
+				}
+				NO_DBG_PRINT_BLOCK(aucPapx, tPapxLen);
+				tList.sLeftIndent =
+					sGetLeftIndent(aucPapx, tPapxLen);
+				aucPapx = xfree(aucPapx);
+			}
+			ulStart += tPapxLen;
+			ucChpxLen = ucGetByte(24, aucLvlfInfo);
+			ulStart += (ULONG)ucChpxLen;
+			/* Read the length of the XString */
+			if (!bReadBuffer(pFile, ulTableStartBlock,
+					aulBlockDepot, tBlockDepotLen,
+					tBlockSize, aucXst,
+					ulStart, sizeof(aucXst))) {
+				aucLstfInfo = xfree(aucLstfInfo);
+				return;
+			}
+			NO_DBG_PRINT_BLOCK(aucXst, sizeof(aucXst));
+			tXstLen = (size_t)usGetWord(0, aucXst);
+			ulStart += sizeof(aucXst);
+			if (tXstLen == 0) {
+				tList.usListChar = DEFAULT_LISTCHAR;
+				vAdd2ListInfoList(ulListID,
+						usIstd,
+						ucListLevel,
+						&tList);
+				continue;
+			}
+			tXstLen *= 2;	/* Length in chars to length in bytes */
+			aucXString = xmalloc(tXstLen);
+			/* Read the XString */
+			if (!bReadBuffer(pFile, ulTableStartBlock,
+					aulBlockDepot, tBlockDepotLen,
+					tBlockSize, aucXString,
+					ulStart, tXstLen)) {
+				aucXString = xfree(aucXString);
+				aucLstfInfo = xfree(aucLstfInfo);
+				return;
+			}
+			NO_DBG_PRINT_BLOCK(aucXString, tXstLen);
+			tOff = 0;
+			for (iNums = 6; iNums < 15; iNums++) {
+				ucTmp = ucGetByte(iNums, aucLvlfInfo);
+				if (ucTmp == 0) {
+					break;
+				}
+				tOff = (size_t)ucTmp;
+			}
+			tOff *= 2;	/* Offset in chars to offset in bytes */
+			NO_DBG_DEC(tOff);
+			if (tList.ucNFC == LIST_SPECIAL ||
+			    tList.ucNFC == LIST_BULLETS) {
+				tList.usListChar = usGetWord(0, aucXString);
+			} else if (tOff != 0 && tOff < tXstLen) {
+				tList.usListChar = usGetWord(tOff, aucXString);
+			} else {
+				tList.usListChar = DEFAULT_LISTCHAR;
+			}
+			vAdd2ListInfoList(ulListID,
+					usIstd,
+					ucListLevel,
+					&tList);
+			ulStart += tXstLen;
+			aucXString = xfree(aucXString);
+		}
+	}
+	aucLstfInfo = xfree(aucLstfInfo);
+} /* end of vGet8LstInfo */
+
 /*
  * Build the lists with Paragraph Information for Word 8/9/10 files
  */
@@ -488,7 +807,7 @@ vGet8PapInfo(FILE *pFile, const pps_info_type *pPPS,
 	ULONG	ulBeginParfInfo;
 	ULONG	ulTableSize, ulTableStartBlock;
 	size_t	tParfInfoLen, tBlockDepotLen;
-	size_t	tBlockSize, tOffset, tSize, tLen;
+	size_t	tBlockSize, tOffset, tLen;
 	int	iIndex, iIndex2, iRun, iFodo, iLen;
 	row_info_enum	eRowInfo;
 	USHORT	usDocStatus, usIstd;
@@ -522,16 +841,17 @@ vGet8PapInfo(FILE *pFile, const pps_info_type *pPPS,
 	}
 	DBG_HEX(ulTableSize);
 	if (ulTableSize < MIN_SIZE_FOR_BBD_USE) {
-	  	/* Use the Small Block Depot */
+		/* Use the Small Block Depot */
 		aulBlockDepot = aulSBD;
 		tBlockDepotLen = tSBDLen;
 		tBlockSize = SMALL_BLOCK_SIZE;
 	} else {
-	  	/* Use the Big Block Depot */
+		/* Use the Big Block Depot */
 		aulBlockDepot = aulBBD;
 		tBlockDepotLen = tBBDLen;
 		tBlockSize = BIG_BLOCK_SIZE;
 	}
+
 	aucBuffer = xmalloc(tParfInfoLen);
 	if (!bReadBuffer(pFile, ulTableStartBlock,
 			aulBlockDepot, tBlockDepotLen, tBlockSize,
@@ -542,8 +862,7 @@ vGet8PapInfo(FILE *pFile, const pps_info_type *pPPS,
 	NO_DBG_PRINT_BLOCK(aucBuffer, tParfInfoLen);
 
 	tLen = (tParfInfoLen / 4 - 1) / 2;
-	tSize = tLen * sizeof(ULONG);
-	aulParfPage = xmalloc(tSize);
+	aulParfPage = xcalloc(tLen, sizeof(ULONG));
 	for (iIndex = 0, tOffset = (tLen + 1) * 4;
 	     iIndex < (int)tLen;
 	     iIndex++, tOffset += 4) {
@@ -641,36 +960,39 @@ vGet8FontInfo(int iFodo, USHORT usIstd,
 {
 	long	lTmp;
 	int	iFodoOff, iInfoLen;
-	USHORT	usTmp;
-	short	sTmp;
+	USHORT	usFtc0, usFtc1, usFtc2, usTmp;
 	UCHAR	ucTmp;
 
 	fail(iFodo < 0 || aucGrpprl == NULL || pFont == NULL);
 
+	usFtc0 = USHRT_MAX;
+	usFtc1 = USHRT_MAX;
+	usFtc2 = USHRT_MAX;
+
 	iFodoOff = 0;
 	while (iBytes >= iFodoOff + 2) {
 		switch (usGetWord(iFodo + iFodoOff, aucGrpprl)) {
 		case 0x0800:	/* fRMarkDel */
 			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			if (ucTmp == 0) {
-				pFont->ucFontstyle &= ~FONT_MARKDEL;
+				pFont->usFontStyle &= ~FONT_MARKDEL;
 			} else {
-				pFont->ucFontstyle |= FONT_MARKDEL;
+				pFont->usFontStyle |= FONT_MARKDEL;
 			}
 			break;
 		case 0x0835:	/* fBold */
 			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			switch (ucTmp) {
 			case   0:	/* Unset */
-				pFont->ucFontstyle &= ~FONT_BOLD;
+				pFont->usFontStyle &= ~FONT_BOLD;
 				break;
 			case   1:	/* Set */
-				pFont->ucFontstyle |= FONT_BOLD;
+				pFont->usFontStyle |= FONT_BOLD;
 				break;
 			case 128:	/* Unchanged */
 				break;
 			case 129:	/* Negation */
-				pFont->ucFontstyle ^= FONT_BOLD;
+				pFont->usFontStyle ^= FONT_BOLD;
 				break;
 			default:
 				DBG_DEC(ucTmp);
@@ -682,15 +1004,15 @@ vGet8FontInfo(int iFodo, USHORT usIstd,
 			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			switch (ucTmp) {
 			case   0:	/* Unset */
-				pFont->ucFontstyle &= ~FONT_ITALIC;
+				pFont->usFontStyle &= ~FONT_ITALIC;
 				break;
 			case   1:	/* Set */
-				pFont->ucFontstyle |= FONT_ITALIC;
+				pFont->usFontStyle |= FONT_ITALIC;
 				break;
 			case 128:	/* Unchanged */
 				break;
 			case 129:	/* Negation */
-				pFont->ucFontstyle ^= FONT_ITALIC;
+				pFont->usFontStyle ^= FONT_ITALIC;
 				break;
 			default:
 				DBG_DEC(ucTmp);
@@ -702,15 +1024,15 @@ vGet8FontInfo(int iFodo, USHORT usIstd,
 			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			switch (ucTmp) {
 			case   0:	/* Unset */
-				pFont->ucFontstyle &= ~FONT_STRIKE;
+				pFont->usFontStyle &= ~FONT_STRIKE;
 				break;
 			case   1:	/* Set */
-				pFont->ucFontstyle |= FONT_STRIKE;
+				pFont->usFontStyle |= FONT_STRIKE;
 				break;
 			case 128:	/* Unchanged */
 				break;
 			case 129:	/* Negation */
-				pFont->ucFontstyle ^= FONT_STRIKE;
+				pFont->usFontStyle ^= FONT_STRIKE;
 				break;
 			default:
 				DBG_DEC(ucTmp);
@@ -722,15 +1044,15 @@ vGet8FontInfo(int iFodo, USHORT usIstd,
 			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			switch (ucTmp) {
 			case   0:	/* Unset */
-				pFont->ucFontstyle &= ~FONT_SMALL_CAPITALS;
+				pFont->usFontStyle &= ~FONT_SMALL_CAPITALS;
 				break;
 			case   1:	/* Set */
-				pFont->ucFontstyle |= FONT_SMALL_CAPITALS;
+				pFont->usFontStyle |= FONT_SMALL_CAPITALS;
 				break;
 			case 128:	/* Unchanged */
 				break;
 			case 129:	/* Negation */
-				pFont->ucFontstyle ^= FONT_SMALL_CAPITALS;
+				pFont->usFontStyle ^= FONT_SMALL_CAPITALS;
 				break;
 			default:
 				DBG_DEC(ucTmp);
@@ -742,15 +1064,15 @@ vGet8FontInfo(int iFodo, USHORT usIstd,
 			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			switch (ucTmp) {
 			case   0:	/* Unset */
-				pFont->ucFontstyle &= ~FONT_CAPITALS;
+				pFont->usFontStyle &= ~FONT_CAPITALS;
 				break;
 			case   1:	/* Set */
-				pFont->ucFontstyle |= FONT_CAPITALS;
+				pFont->usFontStyle |= FONT_CAPITALS;
 				break;
 			case 128:	/* Unchanged */
 				break;
 			case 129:	/* Negation */
-				pFont->ucFontstyle ^= FONT_CAPITALS;
+				pFont->usFontStyle ^= FONT_CAPITALS;
 				break;
 			default:
 				DBG_DEC(ucTmp);
@@ -762,15 +1084,15 @@ vGet8FontInfo(int iFodo, USHORT usIstd,
 			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			switch (ucTmp) {
 			case   0:	/* Unset */
-				pFont->ucFontstyle &= ~FONT_HIDDEN;
+				pFont->usFontStyle &= ~FONT_HIDDEN;
 				break;
 			case   1:	/* Set */
-				pFont->ucFontstyle |= FONT_HIDDEN;
+				pFont->usFontStyle |= FONT_HIDDEN;
 				break;
 			case 128:	/* Unchanged */
 				break;
 			case 129:	/* Negation */
-				pFont->ucFontstyle ^= FONT_HIDDEN;
+				pFont->usFontStyle ^= FONT_HIDDEN;
 				break;
 			default:
 				DBG_DEC(ucTmp);
@@ -779,8 +1101,8 @@ vGet8FontInfo(int iFodo, USHORT usIstd,
 			}
 			break;
 		case 0x2a32:	/* cDefault */
-			pFont->ucFontstyle &= FONT_HIDDEN;
-			pFont->ucFontcolor = FONT_COLOR_DEFAULT;
+			pFont->usFontStyle &= FONT_HIDDEN;
+			pFont->ucFontColor = FONT_COLOR_DEFAULT;
 			break;
 		case 0x2a33:	/* cPlain */
 			DBG_MSG("2a33: cPlain");
@@ -789,64 +1111,65 @@ vGet8FontInfo(int iFodo, USHORT usIstd,
 		case 0x2a3e:	/* cKul */
 			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			if (ucTmp == 0 || ucTmp == 5) {
-				pFont->ucFontstyle &= ~FONT_UNDERLINE;
+				pFont->usFontStyle &= ~FONT_UNDERLINE;
 			} else {
 				NO_DBG_MSG("Underline text");
-				pFont->ucFontstyle |= FONT_UNDERLINE;
+				pFont->usFontStyle |= FONT_UNDERLINE;
 				if (ucTmp == 6) {
 					DBG_MSG("Bold text");
-					pFont->ucFontstyle |= FONT_BOLD;
+					pFont->usFontStyle |= FONT_BOLD;
 				}
 			}
 			break;
 		case 0x2a42:	/* cIco */
-			pFont->ucFontcolor =
+			pFont->ucFontColor =
 				ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
-			NO_DBG_DEC(pFont->ucFontcolor);
+			NO_DBG_DEC(pFont->ucFontColor);
 			break;
 		case 0x2a48:	/* cIss */
 			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			ucTmp &= 0x07;
-			DBG_MSG_C(ucTmp == 1, "Superscript");
-			DBG_MSG_C(ucTmp == 2, "Subscript");
+			if (ucTmp == 1) {
+				pFont->usFontStyle |= FONT_SUPERSCRIPT;
+				NO_DBG_MSG("Superscript");
+			} else if (ucTmp == 2) {
+				pFont->usFontStyle |= FONT_SUBSCRIPT;
+				NO_DBG_MSG("Subscript");
+			}
 			break;
 		case 0x4a30:	/* cIstd */
 			usTmp = usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
 			NO_DBG_DEC(usTmp);
 			break;
 		case 0x4a43:	/* cHps */
-			usTmp = usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
-			if (usTmp > (USHORT)SHRT_MAX) {
-				pFont->sFontsize = SHRT_MAX;
-			} else {
-				pFont->sFontsize = (short)usTmp;
-			}
-			NO_DBG_DEC(pFont->sFontsize);
+			pFont->usFontSize =
+				usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
+			NO_DBG_DEC(pFont->usFontSize);
 			break;
-		case 0x4a51:	/* cFtc */
-			usTmp = usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
-			if (usTmp <= (USHORT)UCHAR_MAX) {
-				pFont->ucFontnumber = (UCHAR)usTmp;
-			} else {
-				pFont->ucFontnumber = 0;
-			}
+		case 0x4a4f:	/* cFtc0 */
+			usFtc0 = usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
+			break;
+		case 0x4a50:	/* cFtc1 */
+			usFtc1 = usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
+			break;
+		case 0x4a51:	/* cFtc2 */
+			usFtc2 = usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
 			break;
 		case 0xca4a:	/* cHps */
-			sTmp = (short)usGetWord(
-					iFodo + iFodoOff + 2, aucGrpprl);
-			lTmp = (long)pFont->sFontsize + (long)sTmp;
+			usTmp = usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
+			lTmp = (long)pFont->usFontSize + (long)usTmp;
 			if (lTmp < 8) {
-				pFont->sFontsize = 8;
+				pFont->usFontSize = 8;
 			} else if (lTmp > 32766) {
-				pFont->sFontsize = 32766;
+				pFont->usFontSize = 32766;
 			} else {
-				pFont->sFontsize = (short)lTmp;
+				pFont->usFontSize = (USHORT)lTmp;
 			}
 			break;
 		case 0xea3f:	/* cHps, cHpsPos */
 			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			if (ucTmp != 0) {
-				pFont->sFontsize = (short)ucTmp;
+				pFont->usFontSize = (USHORT)ucTmp;
 			}
 			DBG_DEC(ucTmp);
 			break;
@@ -857,6 +1180,36 @@ vGet8FontInfo(int iFodo, USHORT usIstd,
 		fail(iInfoLen <= 0);
 		iFodoOff += iInfoLen;
 	}
+
+	/* Combine the Ftc's to a FontNumber */
+	NO_DBG_DEC_C(usFtc0 != USHRT_MAX, usFtc0);
+	NO_DBG_DEC_C(usFtc2 != USHRT_MAX, usFtc2);
+	NO_DBG_DEC_C(usFtc1 != USHRT_MAX, usFtc1);
+	if (usFtc0 <= 0x7fff) {
+		if (usFtc0 <= (USHORT)UCHAR_MAX) {
+			pFont->ucFontNumber = (UCHAR)usFtc0;
+		} else {
+			DBG_DEC(usFtc0);
+			DBG_FIXME();
+			pFont->ucFontNumber = 0;
+		}
+	} else if (usFtc2 <= 0x7fff) {
+		if (usFtc2 <= (USHORT)UCHAR_MAX) {
+			pFont->ucFontNumber = (UCHAR)usFtc2;
+		} else {
+			DBG_DEC(usFtc2);
+			DBG_FIXME();
+			pFont->ucFontNumber = 0;
+		}
+	} else if (usFtc1 <= 0x7fff) {
+		if (usFtc1 <= (USHORT)UCHAR_MAX) {
+			pFont->ucFontNumber = (UCHAR)usFtc1;
+		} else {
+			DBG_DEC(usFtc1);
+			DBG_FIXME();
+			pFont->ucFontNumber = 0;
+		}
+	}
 } /* end of vGet8FontInfo */
 
 /*
@@ -926,7 +1279,7 @@ vGet8ChrInfo(FILE *pFile, const pps_info_type *pPPS,
 	ULONG	ulFileOffset, ulCharPos, ulBeginCharInfo;
 	ULONG	ulTableSize, ulTableStartBlock;
 	size_t	tCharInfoLen, tBlockDepotLen;
-	size_t	tOffset, tBlockSize, tSize, tLen;
+	size_t	tOffset, tBlockSize, tLen;
 	int	iIndex, iIndex2, iRun, iFodo, iLen;
 	USHORT	usDocStatus, usIstd;
 	UCHAR	aucFpage[BIG_BLOCK_SIZE];
@@ -959,12 +1312,12 @@ vGet8ChrInfo(FILE *pFile, const pps_info_type *pPPS,
 	}
 	DBG_HEX(ulTableSize);
 	if (ulTableSize < MIN_SIZE_FOR_BBD_USE) {
-	  	/* Use the Small Block Depot */
+		/* Use the Small Block Depot */
 		aulBlockDepot = aulSBD;
 		tBlockDepotLen = tSBDLen;
 		tBlockSize = SMALL_BLOCK_SIZE;
 	} else {
-	  	/* Use the Big Block Depot */
+		/* Use the Big Block Depot */
 		aulBlockDepot = aulBBD;
 		tBlockDepotLen = tBBDLen;
 		tBlockSize = BIG_BLOCK_SIZE;
@@ -979,8 +1332,7 @@ vGet8ChrInfo(FILE *pFile, const pps_info_type *pPPS,
 	NO_DBG_PRINT_BLOCK(aucBuffer, tCharInfoLen);
 
 	tLen = (tCharInfoLen / 4 - 1) / 2;
-	tSize = tLen * sizeof(ULONG);
-	aulCharPage = xmalloc(tSize);
+	aulCharPage = xcalloc(tLen, sizeof(ULONG));
 	for (iIndex = 0, tOffset = (tLen + 1) * 4;
 	     iIndex < (int)tLen;
 	     iIndex++, tOffset += 4) {
