@@ -1,6 +1,6 @@
 /*
  * riscos.c
- * Copyright (C) 2001 A.J. van Os; Released under GPL
+ * Copyright (C) 2001,2002 A.J. van Os; Released under GPL
  *
  * Description:
  * RISC OS only functions
@@ -144,12 +144,12 @@ bMakeDirectory(const char *szFilename)
 } /* end of bMakeDirectory */
 
 /*
- * bISO_8859_1_IsCurrent
- * This function checks whether ISO_8859_1 (aka Latin1) is the current
- * character set.
+ * iReadCurrentAlphabetNumber
+ * This function reads the current Alphabet number.
+ * Returns the current Alphabet number when successful, otherwise -1
  */
-BOOL
-bISO_8859_1_IsCurrent(void)
+int
+iReadCurrentAlphabetNumber(void)
 {
 	_kernel_swi_regs	regs;
 	_kernel_oserror		*e;
@@ -159,11 +159,11 @@ bISO_8859_1_IsCurrent(void)
 	regs.r[1] = 127;
 	e = _kernel_swi(OS_Byte, &regs, &regs);
 	if (e == NULL) {
-		return regs.r[1] == 101;
+		return regs.r[1];
 	}
 	werr(0, "Read alphabet error %d: %s", e->errnum, e->errmess);
-	return FALSE;
-} /* end of bISO_8859_1_IsCurrent */
+	return -1;
+} /* end of iReadCurrentAlphabetNumber */
 
 /*
  * iGetRiscOsVersion - get the RISC OS version number
@@ -203,12 +203,14 @@ iGetRiscOsVersion(void)
 		return 360;
 	case 0xa7:	/* RISC OS 3.7x */
 		return 370;
+	case 0xa8:	/* RISC OS 4.0x */
+		return 400;
 	default:
-		if (regs.r[1] >= 0xa8 && regs.r[1] <= 0xaf) {
-			/* RISC OS 4.00 and up */
-			return 400;
+		if (regs.r[1] >= 0xa9 && regs.r[1] <= 0xaf) {
+			/* RISC OS 4.10 and up */
+			return 410;
 		}
-		/* Unknow version */
+		/* Unknown version */
 		return 0;
 	}
 } /* end of iGetRiscOsVersion */
