@@ -1,6 +1,6 @@
 /*
  * unix.c
- * Copyright (C) 1998,1999 A.J. van Os
+ * Copyright (C) 1998-2000 A.J. van Os; Released under GPL
  *
  * Description:
  * Unix approximations of RISC-OS functions
@@ -21,10 +21,15 @@ werr(int iFatal, const char *szFormat, ...)
 	va_list tArg;
 
 	va_start(tArg, szFormat);
-	vfprintf(stderr, szFormat, tArg);
+	(void)vfprintf(stderr, szFormat, tArg);
 	va_end(tArg);
 	fprintf(stderr, "\n");
-	if (iFatal != 0) {
+	switch (iFatal) {
+	case 0:		/* The message is just a warning, so no exit */
+		return;
+	case 1:		/* Fatal error with a standard exit */
+		exit(EXIT_FAILURE);
+	default:	/* Fatal error with a non-standard exit */
 		exit(iFatal);
 	}
 } /* end of werr */
