@@ -1,6 +1,6 @@
 /*
  * word2text.c
- * Copyright (C) 1998-2002 A.J. van Os; Released under GPL
+ * Copyright (C) 1998-2003 A.J. van Os; Released under GNU GPL
  *
  * Description:
  * MS Word to text functions
@@ -36,15 +36,15 @@
 
 #if defined(__riscos)
 /* Length of the document in characters */
-static long	lDocumentLength;
+static ULONG	ulDocumentLength;
 /* Number of characters processed so far */
-static long	lCharCounter;
+static ULONG	ulCharCounter;
 static int	iCurrPct, iPrevPct;
 #endif /* __riscos */
 /* The document is in the format belonging to this version of Word */
 static int	iWordVersion = -1;
 /* Special treatment for files from Word 6 on an Apple Macintosh */
-static BOOL	bWord6MacFile = FALSE;
+static BOOL	bOldMacFile = FALSE;
 /* Section Information */
 static const section_block_type	*pSection = NULL;
 static const section_block_type	*pSectionNext = NULL;
@@ -79,8 +79,8 @@ static void
 vUpdateCounters(void)
 {
 #if defined(__riscos)
-	lCharCounter++;
-	iCurrPct = (int)((lCharCounter * 100) / lDocumentLength);
+	ulCharCounter++;
+	iCurrPct = (int)((ulCharCounter * 100) / ulDocumentLength);
 	if (iCurrPct != iPrevPct) {
 		visdelay_percent(iCurrPct);
 		iPrevPct = iCurrPct;
@@ -95,18 +95,18 @@ static BOOL
 bOutputContainsText(output_type *pAnchor)
 {
 	output_type	*pCurr;
-	int	iIndex;
+	size_t	tIndex;
 
 	fail(pAnchor == NULL);
 
 	for (pCurr = pAnchor; pCurr != NULL; pCurr = pCurr->pNext) {
 		fail(pCurr->lStringWidth < 0);
-		for (iIndex = 0; iIndex < pCurr->iNextFree; iIndex++) {
-			if (isspace((int)pCurr->szStorage[iIndex])) {
+		for (tIndex = 0; tIndex < pCurr->tNextFree; tIndex++) {
+			if (isspace((int)(UCHAR)pCurr->szStorage[tIndex])) {
 				continue;
 			}
 #if defined(DEBUG)
-			if (pCurr->szStorage[iIndex] == FILLER_CHAR) {
+			if (pCurr->szStorage[tIndex] == FILLER_CHAR) {
 				continue;
 			}
 #endif /* DEBUG */
@@ -143,76 +143,72 @@ vStoreByte(UCHAR ucChar, output_type *pOutput)
 	fail(pOutput == NULL);
 
 	if (ucChar == 0) {
-		pOutput->szStorage[pOutput->iNextFree] = '\0';
+		pOutput->szStorage[pOutput->tNextFree] = '\0';
 		return;
 	}
 
-	while (pOutput->iNextFree + 2 > (int)pOutput->tStorageSize) {
+	while (pOutput->tNextFree + 2 > pOutput->tStorageSize) {
 		pOutput->tStorageSize += EXTENTION_SIZE;
 		pOutput->szStorage = xrealloc(pOutput->szStorage,
 					pOutput->tStorageSize);
 	}
-	pOutput->szStorage[pOutput->iNextFree] = (char)ucChar;
-	pOutput->szStorage[pOutput->iNextFree + 1] = '\0';
-	pOutput->iNextFree++;
+	pOutput->szStorage[pOutput->tNextFree] = (char)ucChar;
+	pOutput->szStorage[pOutput->tNextFree + 1] = '\0';
+	pOutput->tNextFree++;
 } /* end of vStoreByte */
 
+/*
+ * vStoreChar - store a character as one or more bytes
+ */
+static void
+vStoreChar(ULONG ulChar, BOOL bChangeAllowed, output_type *pOutput)
+{
+	char	szResult[4];
+	size_t	tIndex, tLen;
+
+	fail(pOutput == NULL);
+
+	if (tOptions.eEncoding == encoding_utf8 && bChangeAllowed) {
+		DBG_HEX_C(ulChar > 0xffff, ulChar);
+		fail(ulChar > 0xffff);
+		tLen = tUcs2Utf8(ulChar, szResult, sizeof(szResult));
+		for (tIndex = 0; tIndex < tLen; tIndex++) {
+			vStoreByte((UCHAR)szResult[tIndex], pOutput);
+		}
+	} else {
+		DBG_HEX_C(ulChar > 0xff, ulChar);
+		fail(ulChar > 0xff);
+		vStoreByte((UCHAR)ulChar, pOutput);
+		tLen = 1;
+	}
+	pOutput->lStringWidth += lComputeStringWidth(
+				pOutput->szStorage + pOutput->tNextFree - tLen,
+				tLen,
+				pOutput->tFontRef,
+				pOutput->usFontSize);
+} /* end of vStoreChar */
+
 /*
  * vStoreCharacter - store one character
  */
 static void
 vStoreCharacter(ULONG ulChar, output_type *pOutput)
 {
-	int	iLen;
-
-	fail(pOutput == NULL);
-
-	if (tOptions.eEncoding == encoding_utf8) {
-		DBG_HEX_C(ulChar > 0xffff, ulChar);
-		fail(ulChar > 0xffff);
-		if (ulChar < 0x80) {
-			vStoreByte((UCHAR)ulChar, pOutput);
-			iLen = 1;
-		} else if (ulChar < 0x800) {
-			vStoreByte((UCHAR)(0xc0 | ulChar >> 6),
-								pOutput);
-			vStoreByte((UCHAR)(0x80 | (ulChar & 0x3f)),
-								pOutput);
-			iLen = 2;
-		} else {
-			vStoreByte((UCHAR)(0xe0 | ulChar >> 12),
-								pOutput);
-			vStoreByte((UCHAR)(0x80 | (ulChar >> 6 & 0x3f)),
-								pOutput);
-			vStoreByte((UCHAR)(0x80 | (ulChar & 0x3f)),
-								pOutput);
-			iLen = 3;
-		}
-	} else {
-		DBG_HEX_C(ulChar > 0xff, ulChar);
-		fail(ulChar > 0xff);
-		vStoreByte((UCHAR)ulChar, pOutput);
-		iLen = 1;
-	}
-	pOutput->lStringWidth += lComputeStringWidth(
-				pOutput->szStorage + pOutput->iNextFree - iLen,
-				iLen,
-				pOutput->tFontRef,
-				pOutput->sFontsize);
+	vStoreChar(ulChar, TRUE, pOutput);
 } /* end of vStoreCharacter */
 
 /*
  * vStoreString - store a string
  */
 static void
-vStoreString(const char *szString, int iStringLength, output_type *pOutput)
+vStoreString(const char *szString, size_t tStringLength, output_type *pOutput)
 {
-	int	iIndex;
+	size_t	tIndex;
 
-	fail(szString == NULL || iStringLength < 0 || pOutput == NULL);
+	fail(szString == NULL || pOutput == NULL);
 
-	for (iIndex = 0; iIndex < iStringLength; iIndex++) {
-		vStoreCharacter(szString[iIndex], pOutput);
+	for (tIndex = 0; tIndex < tStringLength; tIndex++) {
+		vStoreCharacter((UCHAR)szString[tIndex], pOutput);
 	}
 } /* end of vStoreString */
 
@@ -222,14 +218,14 @@ vStoreString(const char *szString, int iStringLength, output_type *pOutput)
 static void
 vStoreNumberAsDecimal(UINT uiNumber, output_type *pOutput)
 {
-	int	iLen;
-	char	szString[15];
+	size_t	tLen;
+	char	szString[3 * sizeof(UINT) + 1];
 
 	fail(uiNumber == 0);
 	fail(pOutput == NULL);
 
-	iLen = sprintf(szString, "%u", uiNumber);
-	vStoreString(szString, iLen, pOutput);
+	tLen = (size_t)sprintf(szString, "%u", uiNumber);
+	vStoreString(szString, tLen, pOutput);
 } /* end of vStoreNumberAsDecimal */
 
 /*
@@ -238,127 +234,151 @@ vStoreNumberAsDecimal(UINT uiNumber, output_type *pOutput)
 static void
 vStoreNumberAsRoman(UINT uiNumber, output_type *pOutput)
 {
-	int	iLen;
+	size_t	tLen;
 	char	szString[15];
 
 	fail(uiNumber == 0);
 	fail(pOutput == NULL);
 
-	iLen = iNumber2Roman(uiNumber, FALSE, szString);
-	vStoreString(szString, iLen, pOutput);
+	tLen = tNumber2Roman(uiNumber, FALSE, szString);
+	vStoreString(szString, tLen, pOutput);
 } /* end of vStoreNumberAsRoman */
 
 /*
  * vStoreStyle - store a style
  */
 static void
-vStoreStyle(output_type *pOutput, const style_block_type *pStyle)
+vStoreStyle(diagram_type *pDiag, output_type *pOutput,
+	const style_block_type *pStyle)
 {
-	int	iLen;
+	size_t	tLen;
 	char	szString[120];
 
+	fail(pDiag == NULL);
 	fail(pOutput == NULL);
 	fail(pStyle == NULL);
 
-	iLen = iStyle2Window(szString, pStyle, pSection);
-	vStoreString(szString, iLen, pOutput);
+	if (tOptions.eConversionType == conversion_xml) {
+		vSetHeaders(pDiag, pStyle->usIstd);
+	} else {
+		tLen = tStyle2Window(szString, pStyle, pSection);
+		vStoreString(szString, tLen, pOutput);
+	}
 } /* end of vStoreStyle */
 
 /*
- * Create an empty line by adding a extra "newline"
+ * vPutIndentation - output the specified amount of indentation
  */
 static void
-vEmptyLine2Diagram(diagram_type *pDiag, draw_fontref tFontRef, short sFontsize)
-{
-	fail(pDiag == NULL);
-	fail(sFontsize < MIN_FONT_SIZE || sFontsize > MAX_FONT_SIZE);
-
-	if (pDiag->lXleft > 0) {
-		/* To the start of the line */
-		vMove2NextLine(pDiag, tFontRef, sFontsize);
-	}
-	/* Empty line */
-	vMove2NextLine(pDiag, tFontRef, sFontsize);
-} /* end of vEmptyLine2Diagram */
-
-/*
- * vPutIndentation - output the given amount of indentation
- */
-static void
-vPutIndentation(diagram_type *pDiag, output_type *pOutput, BOOL bNumPause,
-	UINT uiListNumber, UCHAR ucNFC, char cListCharacter,
-	long lLeftIndentation)
+vPutIndentation(diagram_type *pDiag, output_type *pOutput,
+	BOOL bNoMarks, BOOL bFirstLine,
+	UINT uiListNumber, UCHAR ucNFC, const char *szListChar,
+	long lLeftIndentation, long lLeftIndentation1)
 {
 	long	lWidth;
-	int	iNextFree;
+	size_t	tIndex, tNextFree;
 	char	szLine[30];
 
-	fail(pDiag == NULL || pOutput == NULL);
+	fail(pDiag == NULL);
+	fail(pOutput == NULL);
+	fail(szListChar == NULL);
 	fail(lLeftIndentation < 0);
 
-	if (lLeftIndentation <= 0) {
+	if (tOptions.eConversionType == conversion_xml) {
+		/* XML does its own indentation at rendering time */
 		return;
 	}
-	if (bNumPause) {
+
+	if (bNoMarks) {
+		if (bFirstLine) {
+			lLeftIndentation += lLeftIndentation1;
+		}
+		if (lLeftIndentation < 0) {
+			lLeftIndentation = 0;
+		}
 		vSetLeftIndentation(pDiag, lLeftIndentation);
 		return;
 	}
-	fail(iscntrl((int)cListCharacter));
+	if (lLeftIndentation <= 0) {
+		DBG_HEX_C(ucNFC != 0x00, ucNFC);
+		vSetLeftIndentation(pDiag, 0);
+		return;
+	}
+
+#if defined(DEBUG)
+	if (tOptions.eEncoding == encoding_utf8) {
+		fail(strlen(szListChar) > 3);
+	} else {
+		DBG_HEX_C(iscntrl((int)szListChar[0]), szListChar[0]);
+		fail(iscntrl((int)szListChar[0]));
+		fail(szListChar[1] != '\0');
+	}
+#endif /* DEBUG */
 
 	switch (ucNFC) {
 	case LIST_ARABIC_NUM:
-		iNextFree = sprintf(szLine, "%u", uiListNumber);
+	case LIST_ARABIC_NUM_2:
+	case LIST_ARABIC_NUM_3:
+		tNextFree = (size_t)sprintf(szLine, "%u", uiListNumber);
 		break;
-	case LIST_ROMAN_NUM_UPPER:
-	case LIST_ROMAN_NUM_LOWER:
-		iNextFree = iNumber2Roman(uiListNumber,
-				ucNFC == LIST_ROMAN_NUM_UPPER, szLine);
+	case LIST_UPPER_ROMAN:
+	case LIST_LOWER_ROMAN:
+		tNextFree = tNumber2Roman(uiListNumber,
+				ucNFC == LIST_UPPER_ROMAN, szLine);
 		break;
 	case LIST_UPPER_ALPHA:
 	case LIST_LOWER_ALPHA:
-		iNextFree = iNumber2Alpha(uiListNumber,
+		tNextFree = tNumber2Alpha(uiListNumber,
 				ucNFC == LIST_UPPER_ALPHA, szLine);
 		break;
 	case LIST_ORDINAL_NUM:
 		if (uiListNumber % 10 == 1 && uiListNumber != 11) {
-			iNextFree = sprintf(szLine, "%ust", uiListNumber);
+			tNextFree =
+				(size_t)sprintf(szLine, "%ust", uiListNumber);
 		} else if (uiListNumber % 10 == 2 && uiListNumber != 12) {
-			iNextFree = sprintf(szLine, "%und", uiListNumber);
+			tNextFree =
+				(size_t)sprintf(szLine, "%und", uiListNumber);
 		} else if (uiListNumber % 10 == 3 && uiListNumber != 13) {
-			iNextFree = sprintf(szLine, "%urd", uiListNumber);
+			tNextFree =
+				(size_t)sprintf(szLine, "%urd", uiListNumber);
 		} else {
-			iNextFree = sprintf(szLine, "%uth", uiListNumber);
+			tNextFree =
+				(size_t)sprintf(szLine, "%uth", uiListNumber);
 		}
 		break;
+	case LIST_SPECIAL:
 	case LIST_BULLETS:
-		iNextFree = 0;
+		tNextFree = 0;
 		break;
 	default:
 		DBG_DEC(ucNFC);
 		DBG_FIXME();
-		iNextFree = sprintf(szLine, "%u", uiListNumber);
+		tNextFree = (size_t)sprintf(szLine, "%u", uiListNumber);
 		break;
 	}
-	szLine[iNextFree++] = cListCharacter;
-	szLine[iNextFree++] = ' ';
-	szLine[iNextFree] = '\0';
-	lWidth = lComputeStringWidth(szLine, iNextFree,
-				pOutput->tFontRef, pOutput->sFontsize);
+	tNextFree += (size_t)sprintf(szLine + tNextFree, "%.3s", szListChar);
+	szLine[tNextFree++] = ' ';
+	szLine[tNextFree] = '\0';
+	lWidth = lComputeStringWidth(szLine, tNextFree,
+				pOutput->tFontRef, pOutput->usFontSize);
 	lLeftIndentation -= lWidth;
-	if (lLeftIndentation > 0) {
-		vSetLeftIndentation(pDiag, lLeftIndentation);
+	if (lLeftIndentation < 0) {
+		lLeftIndentation = 0;
+	}
+	vSetLeftIndentation(pDiag, lLeftIndentation);
+	for (tIndex = 0; tIndex < tNextFree; tIndex++) {
+		vStoreChar((UCHAR)szLine[tIndex], FALSE, pOutput);
 	}
-	vStoreString(szLine, iNextFree, pOutput);
 } /* end of vPutIndentation */
 
 /*
- * vPutNoteSeparator - output a note separator
+ * vPutSeparatorLine - output a separator line
  *
- * A note separator is a horizontal line two inches long.
+ * A separator line is a horizontal line two inches long.
  * Two inches equals 144000 millipoints.
  */
 static void
-vPutNoteSeparator(output_type *pOutput)
+vPutSeparatorLine(output_type *pOutput)
 {
 	long	lCharWidth;
 	int	iCounter, iChars;
@@ -369,24 +389,26 @@ vPutNoteSeparator(output_type *pOutput)
 	szOne[0] = OUR_EM_DASH;
 	szOne[1] = '\0';
 	lCharWidth = lComputeStringWidth(szOne, 1,
-				pOutput->tFontRef, pOutput->sFontsize);
-	DBG_DEC(lCharWidth);
+				pOutput->tFontRef, pOutput->usFontSize);
+	NO_DBG_DEC(lCharWidth);
 	iChars = (int)((144000 + lCharWidth / 2) / lCharWidth);
-	DBG_DEC(iChars);
+	NO_DBG_DEC(iChars);
 	for (iCounter = 0; iCounter < iChars; iCounter++) {
-		vStoreCharacter(OUR_EM_DASH, pOutput);
+		vStoreCharacter((ULONG)(UCHAR)OUR_EM_DASH, pOutput);
 	}
-} /* end of vPutNoteSeparator */
+} /* end of vPutSeparatorLine */
 
 /*
+ * pStartNextOutput - start the next output record
  *
+ * returns a pointer to the next record
  */
 static output_type *
 pStartNextOutput(output_type *pCurrent)
 {
 	output_type	*pNew;
 
-	if (pCurrent->iNextFree == 0) {
+	if (pCurrent->tNextFree == 0) {
 		/* The current record is empty, re-use */
 		fail(pCurrent->szStorage[0] != '\0');
 		fail(pCurrent->lStringWidth != 0);
@@ -398,12 +420,12 @@ pStartNextOutput(output_type *pCurrent)
 	pNew->tStorageSize = INITIAL_SIZE;
 	pNew->szStorage = xmalloc(pNew->tStorageSize);
 	pNew->szStorage[0] = '\0';
-	pNew->iNextFree = 0;
+	pNew->tNextFree = 0;
 	pNew->lStringWidth = 0;
-	pNew->iColor = FONT_COLOR_DEFAULT;
-	pNew->ucFontstyle = FONT_REGULAR;
+	pNew->ucFontColor = FONT_COLOR_DEFAULT;
+	pNew->usFontStyle = FONT_REGULAR;
 	pNew->tFontRef = (draw_fontref)0;
-	pNew->sFontsize = DEFAULT_FONT_SIZE;
+	pNew->usFontSize = DEFAULT_FONT_SIZE;
 	pNew->pPrev = pCurrent;
 	pNew->pNext = NULL;
 	return pNew;
@@ -416,25 +438,24 @@ static output_type *
 pStartNewOutput(output_type *pAnchor, output_type *pLeftOver)
 {
 	output_type	*pCurr, *pNext;
-	int		iColor;
-	short		sFontsize;
+	USHORT		usFontStyle, usFontSize;
 	draw_fontref	tFontRef;
-	UCHAR	ucFontstyle;
+	UCHAR		ucFontColor;
 
-	iColor = FONT_COLOR_DEFAULT;
-	ucFontstyle = FONT_REGULAR;
+	ucFontColor = FONT_COLOR_DEFAULT;
+	usFontStyle = FONT_REGULAR;
 	tFontRef = (draw_fontref)0;
-	sFontsize = DEFAULT_FONT_SIZE;
+	usFontSize = DEFAULT_FONT_SIZE;
 	/* Free the old output space */
 	pCurr = pAnchor;
 	while (pCurr != NULL) {
 		pNext = pCurr->pNext;
 		pCurr->szStorage = xfree(pCurr->szStorage);
 		if (pCurr->pNext == NULL) {
-			iColor = pCurr->iColor;
-			ucFontstyle = pCurr->ucFontstyle;
+			ucFontColor = pCurr->ucFontColor;
+			usFontStyle = pCurr->usFontStyle;
 			tFontRef = pCurr->tFontRef;
-			sFontsize = pCurr->sFontsize;
+			usFontSize = pCurr->usFontSize;
 		}
 		pCurr = xfree(pCurr);
 		pCurr = pNext;
@@ -445,12 +466,12 @@ pStartNewOutput(output_type *pAnchor, output_type *pLeftOver)
 		pLeftOver->tStorageSize = INITIAL_SIZE;
 		pLeftOver->szStorage = xmalloc(pLeftOver->tStorageSize);
 		pLeftOver->szStorage[0] = '\0';
-		pLeftOver->iNextFree = 0;
+		pLeftOver->tNextFree = 0;
 		pLeftOver->lStringWidth = 0;
-		pLeftOver->iColor = iColor;
-		pLeftOver->ucFontstyle = ucFontstyle;
+		pLeftOver->ucFontColor = ucFontColor;
+		pLeftOver->usFontStyle = usFontStyle;
 		pLeftOver->tFontRef = tFontRef;
-		pLeftOver->sFontsize = sFontsize;
+		pLeftOver->usFontSize = usFontSize;
 		pLeftOver->pPrev = NULL;
 		pLeftOver->pNext = NULL;
 	}
@@ -459,7 +480,7 @@ pStartNewOutput(output_type *pAnchor, output_type *pLeftOver)
 } /* end of pStartNewOutput */
 
 /*
- * ulGetChar - get the next character from the given list
+ * ulGetChar - get the next character from the specified list
  *
  * returns the next character of EOF
  */
@@ -542,8 +563,9 @@ ulGetChar(FILE *pFile, list_id_enum eListID)
 		ulChar = ulTranslateCharacters(usChar,
 					ulFileOffset,
 					iWordVersion,
+					tOptions.eConversionType,
 					tOptions.eEncoding,
-					bWord6MacFile);
+					bOldMacFile);
 		if (ulChar == IGNORE_CHARACTER) {
 			continue;
 		}
@@ -570,51 +592,53 @@ ulGetChar(FILE *pFile, list_id_enum eListID)
 } /* end of ulGetChar */
 
 /*
- * bWord2Text - translate Word to text or Postcript
+ * bWordDecryptor - translate Word to text or PostScript
  *
  * returns TRUE when succesful, otherwise FALSE
  */
 BOOL
-bWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
+bWordDecryptor(FILE *pFile, long lFilesize, diagram_type *pDiag)
 {
 	imagedata_type	tImage;
 	const style_block_type	*pStyleTmp;
 	const font_block_type	*pFontTmp;
+	const char	*szListChar;
 	output_type	*pAnchor, *pOutput, *pLeftOver;
 	ULONG	ulChar;
 	long	lBeforeIndentation, lAfterIndentation;
-	long	lLeftIndentation, lRightIndentation;
+	long	lLeftIndentation, lLeftIndentation1, lRightIndentation;
 	long	lWidthCurr, lWidthMax, lDefaultTabWidth, lTmp;
 	list_id_enum 	eListID;
 	image_info_enum	eRes;
-	UINT	uiListNumber, uiFootnoteNumber, uiEndnoteNumber;
-	BOOL	bWasTableRow, bTableFontClosed, bWasInList, bWasEndOfParagraph;
-	BOOL	bNumPause, bAllCapitals, bHiddenText, bSuccess;
-	short	sFontsize;
-	UCHAR	ucFontnumber, ucFontcolor, ucTmp;
-	UCHAR	ucFontstyle, ucFontstyleMinimal;
+	UINT	uiFootnoteNumber, uiEndnoteNumber, uiTmp;
+	int	iListSeqNumber;
+	BOOL	bWasTableRow, bTableFontClosed, bWasEndOfParagraph;
+	BOOL	bInList, bWasInList, bNoMarks, bFirstLine;
+	BOOL	bAllCapitals, bHiddenText, bMarkDelText, bSuccess;
+	USHORT	usListNumber;
+	USHORT	usFontStyle, usFontStyleMinimal, usFontSize, usTmp;
+	UCHAR	ucFontNumber, ucFontColor;
 	UCHAR	ucNFC, ucAlignment;
-	char	cListChar;
 
 	fail(pFile == NULL || lFilesize <= 0 || pDiag == NULL);
 
-	DBG_MSG("bWord2Text");
+	DBG_MSG("bWordDecryptor");
 
 	iWordVersion = iInitDocument(pFile, lFilesize);
 	if (iWordVersion < 0) {
 		DBG_DEC(iWordVersion);
 		return FALSE;
 	}
-	vAddFonts2Diagram(pDiag);
+	vPrologue2(pDiag, iWordVersion);
 
 	/* Initialisation */
 #if defined(__riscos)
-	lCharCounter = 0;
+	ulCharCounter = 0;
 	iCurrPct = 0;
 	iPrevPct = -1;
-	lDocumentLength = (long)ulGetDocumentLength();
+	ulDocumentLength = ulGetDocumentLength();
 #endif /* __riscos */
-	bWord6MacFile = bIsWord6MacFile();
+	bOldMacFile = bIsOldMacFile();
 	pSection = pGetSectionInfo(NULL, 0);
 	pSectionNext = pSection;
 	lDefaultTabWidth = lGetDefaultTabWidth();
@@ -631,38 +655,53 @@ bWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 	vResetStyles();
 	pStyleInfo = pGetNextStyleInfoListItem(NULL);
 	bStartStyle = FALSE;
+	bInList = FALSE;
 	bWasInList = FALSE;
+	iListSeqNumber = 0;
 	usIstdNext = ISTD_NORMAL;
 	pAnchor = NULL;
 	pFontInfo = pGetNextFontInfoListItem(NULL);
 	DBG_HEX_C(pFontInfo != NULL, pFontInfo->ulFileOffset);
 	DBG_MSG_C(pFontInfo == NULL, "No fonts at all");
 	bStartFont = FALSE;
-	ucFontnumber = 0;
-	ucFontstyleMinimal = FONT_REGULAR;
-	ucFontstyle = FONT_REGULAR;
-	sFontsize = DEFAULT_FONT_SIZE;
-	ucFontcolor = FONT_COLOR_DEFAULT;
+	ucFontNumber = 0;
+	usFontStyleMinimal = FONT_REGULAR;
+	usFontStyle = FONT_REGULAR;
+	usFontSize = DEFAULT_FONT_SIZE;
+	ucFontColor = FONT_COLOR_DEFAULT;
 	pAnchor = pStartNewOutput(pAnchor, NULL);
 	pOutput = pAnchor;
-	pOutput->iColor = (int)ucFontcolor;
-	pOutput->ucFontstyle = ucFontstyle;
-	pOutput->tFontRef = tOpenFont(ucFontnumber, ucFontstyle, sFontsize);
-	pOutput->sFontsize = sFontsize;
+	pOutput->ucFontColor = ucFontColor;
+	pOutput->usFontStyle = usFontStyle;
+	pOutput->tFontRef = tOpenFont(ucFontNumber, usFontStyle, usFontSize);
+	pOutput->usFontSize = usFontSize;
 	bTableFontClosed = TRUE;
 	lBeforeIndentation = 0;
 	lAfterIndentation = 0;
 	lLeftIndentation = 0;
+	lLeftIndentation1 = 0;
 	lRightIndentation = 0;
 	bWasEndOfParagraph = TRUE;
-	bNumPause = TRUE;
+	bNoMarks = TRUE;
+	bFirstLine = TRUE;
 	ucNFC = LIST_BULLETS;
-	cListChar = OUR_BULLET;
-	uiListNumber = 0;
+	vGetOptions(&tOptions);
+	if (pStyleInfo != NULL) {
+		szListChar = pStyleInfo->szListChar;
+		pStyleTmp = pStyleInfo;
+	} else {
+		if (tStyleNext.szListChar[0] == '\0') {
+			vGetBulletValue(tOptions.eConversionType,
+				tOptions.eEncoding, tStyleNext.szListChar, 4);
+		}
+		szListChar = tStyleNext.szListChar;
+		pStyleTmp = &tStyleNext;
+	}
+	usListNumber = 0;
 	ucAlignment = ALIGNMENT_LEFT;
 	bAllCapitals = FALSE;
 	bHiddenText = FALSE;
-	vGetOptions(&tOptions);
+	bMarkDelText = FALSE;
 	fail(tOptions.iParagraphBreak < 0);
 	if (tOptions.iParagraphBreak == 0) {
 		lWidthMax = LONG_MAX;
@@ -692,7 +731,7 @@ bWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 			case text_list:
 				eListID = footnote_list;
 				if (uiFootnoteNumber != 0) {
-					vPutNoteSeparator(pAnchor);
+					vPutSeparatorLine(pAnchor);
 					OUTPUT_LINE();
 					uiFootnoteNumber = 0;
 				}
@@ -700,12 +739,26 @@ bWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 			case footnote_list:
 				eListID = endnote_list;
 				if (uiEndnoteNumber != 0) {
-					vPutNoteSeparator(pAnchor);
+					vPutSeparatorLine(pAnchor);
 					OUTPUT_LINE();
 					uiEndnoteNumber = 0;
 				}
 				break;
 			case endnote_list:
+				eListID = textbox_list;
+				if (bExistsTextBox()) {
+					vPutSeparatorLine(pAnchor);
+					OUTPUT_LINE();
+				}
+				break;
+			case textbox_list:
+				eListID = hdrtextbox_list;
+				if (bExistsHdrTextBox()) {
+					vPutSeparatorLine(pAnchor);
+					OUTPUT_LINE();
+				}
+				break;
+			case hdrtextbox_list:
 			default:
 				eListID = end_of_lists;
 				break;
@@ -745,14 +798,17 @@ bWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 				 * proportional fonts for its tables and we
 				 * only one fixed-width font
 				 */
-				pOutput->sFontsize =
-					(sFontsize <= DEFAULT_FONT_SIZE ?
-					 (DEFAULT_FONT_SIZE * 5 + 3) / 6 :
-					 (sFontsize * 5 + 3) / 6);
+				uiTmp = ((UINT)usFontSize * 5 + 3) / 6;
+				if (uiTmp < MIN_TABLEFONT_SIZE) {
+					uiTmp = MIN_TABLEFONT_SIZE;
+				} else if (uiTmp > MAX_TABLEFONT_SIZE) {
+					uiTmp = MAX_TABLEFONT_SIZE;
+				}
+				pOutput->usFontSize = (USHORT)uiTmp;
 				pOutput->tFontRef =
-					tOpenTableFont(pOutput->sFontsize);
-				pOutput->ucFontstyle = FONT_REGULAR;
-				pOutput->iColor = FONT_COLOR_BLACK;
+					tOpenTableFont(pOutput->usFontSize);
+				pOutput->usFontStyle = FONT_REGULAR;
+				pOutput->ucFontColor = FONT_COLOR_BLACK;
 				bTableFontClosed = FALSE;
 			}
 			bIsTableRow = TRUE;
@@ -770,10 +826,10 @@ bWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 			 * empty line, like the end of a paragraph
 			 */
 			OUTPUT_LINE();
-			vEndOfParagraph2Diagram(pDiag,
+			vEndOfParagraph(pDiag,
 					pOutput->tFontRef,
-					pOutput->sFontsize,
-					(long)pOutput->sFontsize * 600);
+					pOutput->usFontSize,
+					(long)pOutput->usFontSize * 600);
 		}
 
 		switch (ulChar) {
@@ -789,15 +845,13 @@ bWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 				RESET_LINE();
 			}
 			if (ulChar == PAGE_BREAK) {
-				vEndOfPage2Diagram(pDiag,
-						pOutput->tFontRef,
-						pOutput->sFontsize,
-						lAfterIndentation);
+				vEndOfPage(pDiag,
+					lAfterIndentation);
 			} else {
-				vEndOfParagraph2Diagram(pDiag,
-						pOutput->tFontRef,
-						pOutput->sFontsize,
-						lAfterIndentation);
+				vEndOfParagraph(pDiag,
+					pOutput->tFontRef,
+					pOutput->usFontSize,
+					lAfterIndentation);
 			}
 			break;
 		default:
@@ -813,32 +867,34 @@ bWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 			} else {
 				pFontTmp = &tFontNext;
 			}
-			bAllCapitals = bIsCapitals(pFontTmp->ucFontstyle);
-			bHiddenText = bIsHidden(pFontTmp->ucFontstyle);
-			ucTmp = pFontTmp->ucFontstyle &
+			bAllCapitals = bIsCapitals(pFontTmp->usFontStyle);
+			bHiddenText = bIsHidden(pFontTmp->usFontStyle);
+			bMarkDelText = bIsMarkDel(pFontTmp->usFontStyle);
+			usTmp = pFontTmp->usFontStyle &
 				(FONT_BOLD|FONT_ITALIC|FONT_UNDERLINE|
-				 FONT_STRIKE|FONT_MARKDEL);
+				 FONT_STRIKE|FONT_MARKDEL|
+				 FONT_SUPERSCRIPT|FONT_SUBSCRIPT);
 			if (!bIsTableRow &&
-			    (sFontsize != pFontTmp->sFontsize ||
-			    ucFontnumber != pFontTmp->ucFontnumber ||
-			    ucFontstyleMinimal != ucTmp ||
-			    ucFontcolor != pFontTmp->ucFontcolor)) {
+			    (usFontSize != pFontTmp->usFontSize ||
+			     ucFontNumber != pFontTmp->ucFontNumber ||
+			     usFontStyleMinimal != usTmp ||
+			     ucFontColor != pFontTmp->ucFontColor)) {
 				pOutput = pStartNextOutput(pOutput);
 				vCloseFont();
-				pOutput->iColor = (int)pFontTmp->ucFontcolor;
-				pOutput->ucFontstyle = pFontTmp->ucFontstyle;
-				pOutput->sFontsize = pFontTmp->sFontsize;
+				pOutput->ucFontColor = pFontTmp->ucFontColor;
+				pOutput->usFontStyle = pFontTmp->usFontStyle;
+				pOutput->usFontSize = pFontTmp->usFontSize;
 				pOutput->tFontRef = tOpenFont(
-						pFontTmp->ucFontnumber,
-						pFontTmp->ucFontstyle,
-						pFontTmp->sFontsize);
+						pFontTmp->ucFontNumber,
+						pFontTmp->usFontStyle,
+						pFontTmp->usFontSize);
 				fail(!bCheckDoubleLinkedList(pAnchor));
 			}
-			ucFontnumber = pFontTmp->ucFontnumber;
-			sFontsize = pFontTmp->sFontsize;
-			ucFontcolor = pFontTmp->ucFontcolor;
-			ucFontstyle = pFontTmp->ucFontstyle;
-			ucFontstyleMinimal = ucTmp;
+			ucFontNumber = pFontTmp->ucFontNumber;
+			usFontSize = pFontTmp->usFontSize;
+			ucFontColor = pFontTmp->ucFontColor;
+			usFontStyle = pFontTmp->usFontStyle;
+			usFontStyleMinimal = usTmp;
 			if (bStartFont) {
 				/* Get the next font info */
 				pFontInfo = pGetNextFontInfoListItem(pFontInfo);
@@ -851,6 +907,7 @@ bWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 		}
 
 		if (bStartStyle || (bStartStyleNext && ulChar != PAR_END)) {
+			bFirstLine = TRUE;
 			/* Begin of a style found */
 			if (bStartStyle) {
 				/* bStartStyle takes priority */
@@ -860,7 +917,7 @@ bWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 				pStyleTmp = &tStyleNext;
 			}
 			if (!bIsTableRow) {
-				vStoreStyle(pOutput, pStyleTmp);
+				vStoreStyle(pDiag, pOutput, pStyleTmp);
 			}
 			usIstdNext = pStyleTmp->usIstdNext;
 			lBeforeIndentation =
@@ -869,32 +926,32 @@ bWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 				lTwips2MilliPoints(pStyleTmp->usAfterIndent);
 			lLeftIndentation =
 				lTwips2MilliPoints(pStyleTmp->sLeftIndent);
+			lLeftIndentation1 =
+				lTwips2MilliPoints(pStyleTmp->sLeftIndent1);
 			lRightIndentation =
 				lTwips2MilliPoints(pStyleTmp->sRightIndent);
-			bNumPause = !pStyleTmp->bInList ||
-						pStyleTmp->bNumPause;
+			bInList = bStyleImpliesList(pStyleTmp, iWordVersion);
+			bNoMarks = !bInList || pStyleTmp->bNumPause;
 			ucNFC = pStyleTmp->ucNFC;
-			cListChar = (char)pStyleTmp->ucListCharacter;
+			szListChar = pStyleTmp->szListChar;
 			ucAlignment = pStyleTmp->ucAlignment;
-			if (pStyleTmp->bInList) {
-				if (bWasInList) {
-					if (!pStyleTmp->bNumPause) {
-						uiListNumber++;
-					}
-				} else {
-					uiListNumber =
-						(UINT)pStyleTmp->usStartAt;
-				}
-			} else {
-				uiListNumber = 0;
+			if (bInList && !bWasInList) {
+				/* Start of a list */
+				iListSeqNumber++;
+				vStartOfList(pDiag, ucNFC,
+						bWasTableRow && !bIsTableRow);
 			}
-			bWasInList = pStyleTmp->bInList;
+			if (!bInList && bWasInList) {
+				/* End of a list */
+				vEndOfList(pDiag);
+			}
+			bWasInList = bInList;
 			if (bStartStyle) {
 				pStyleInfo =
 					pGetNextStyleInfoListItem(pStyleInfo);
 				NO_DBG_HEX_C(pStyleInfo != NULL,
 						pStyleInfo->ulFileOffset);
-				NO_DBG_MSG_C(pStyleInfo == NULL,
+				DBG_MSG_C(pStyleInfo == NULL,
 						"No more styles");
 			}
 			bStartStyle = FALSE;
@@ -902,20 +959,30 @@ bWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 		}
 
 		if (bWasEndOfParagraph) {
-			vStartOfParagraph2Diagram(pDiag,
-						pOutput->tFontRef,
-						pOutput->sFontsize,
-						lBeforeIndentation);
-			bWasEndOfParagraph = FALSE;
+			vStartOfParagraph1(pDiag, lBeforeIndentation);
 		}
 
 		if (!bIsTableRow &&
 		    lTotalStringWidth(pAnchor) == 0) {
-			vPutIndentation(pDiag, pAnchor, bNumPause,
-					uiListNumber, ucNFC, cListChar,
-					lLeftIndentation);
+			if (!bNoMarks) {
+				usListNumber = usGetListValue(iListSeqNumber,
+							iWordVersion,
+							pStyleTmp);
+			}
+			if (bInList && bFirstLine) {
+				vStartOfListItem(pDiag, bNoMarks);
+			}
+			vPutIndentation(pDiag, pAnchor, bNoMarks, bFirstLine,
+					usListNumber, ucNFC, szListChar,
+					lLeftIndentation, lLeftIndentation1);
+			bFirstLine = FALSE;
 			/* One number or mark per paragraph will do */
-			bNumPause = TRUE;
+			bNoMarks = TRUE;
+		}
+
+		if (bWasEndOfParagraph) {
+			vStartOfParagraph2(pDiag);
+			bWasEndOfParagraph = FALSE;
 		}
 
 		switch (ulChar) {
@@ -969,10 +1036,10 @@ bWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 				break;
 			}
 			OUTPUT_LINE();
-			vEndOfParagraph2Diagram(pDiag,
-						pOutput->tFontRef,
-						pOutput->sFontsize,
-						lAfterIndentation);
+			vEndOfParagraph(pDiag,
+					pOutput->tFontRef,
+					pOutput->usFontSize,
+					lAfterIndentation);
 			bWasEndOfParagraph = TRUE;
 			break;
 		case HARD_RETURN:
@@ -983,9 +1050,8 @@ bWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 			if (bOutputContainsText(pAnchor)) {
 				OUTPUT_LINE();
 			}
-			vEmptyLine2Diagram(pDiag,
-					pOutput->tFontRef,
-					pOutput->sFontsize);
+			vMove2NextLine(pDiag,
+					pOutput->tFontRef, pOutput->usFontSize);
 			break;
 		case PAGE_BREAK:
 		case COLUMN_FEED:
@@ -997,15 +1063,16 @@ bWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 				break;
 			}
 			vStoreCharacter((ULONG)' ', pOutput);
-			vStoreCharacter(TABLE_SEPARATOR_CHAR, pOutput);
+			vStoreCharacter((ULONG)TABLE_SEPARATOR_CHAR, pOutput);
 			break;
 		case TAB:
-			if (bIsTableRow) {
+			if (bIsTableRow ||
+			    tOptions.eConversionType == conversion_xml) {
 				vStoreCharacter((ULONG)' ', pOutput);
 				break;
 			}
 			if (tOptions.iParagraphBreak == 0 &&
-			    !tOptions.bUseOutlineFonts) {
+			    tOptions.eConversionType == conversion_text) {
 				/* No logical lines, so no tab expansion */
 				vStoreCharacter(TAB, pOutput);
 				break;
@@ -1014,7 +1081,7 @@ bWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 			lTmp += lDrawUnits2MilliPoints(pDiag->lXleft);
 			lTmp /= lDefaultTabWidth;
 			do {
-				vStoreCharacter(FILLER_CHAR, pOutput);
+				vStoreCharacter((ULONG)FILLER_CHAR, pOutput);
 				lWidthCurr = lTotalStringWidth(pAnchor);
 				lWidthCurr +=
 					lDrawUnits2MilliPoints(pDiag->lXleft);
@@ -1025,6 +1092,10 @@ bWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 			if (bHiddenText && tOptions.bHideHiddenText) {
 				continue;
 			}
+			if (bMarkDelText &&
+			    tOptions.eConversionType != conversion_ps) {
+				continue;
+			}
 			if (bAllCapitals) {
 				ulChar = ulToUpper(ulChar);
 			}
@@ -1033,15 +1104,17 @@ bWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 		}
 
 		if (bWasTableRow && !bIsTableRow) {
-			/* End of a table, resume normal font */
+			/* End of a table */
+			vEndOfTable(pDiag);
+			/* Resume normal font */
 			NO_DBG_MSG("End of table font");
 			vCloseFont();
 			bTableFontClosed = TRUE;
-			pOutput->iColor = ucFontcolor;
-			pOutput->ucFontstyle = ucFontstyle;
-			pOutput->sFontsize = sFontsize;
-			pOutput->tFontRef =
-				tOpenFont(ucFontnumber, ucFontstyle, sFontsize);
+			pOutput->ucFontColor = ucFontColor;
+			pOutput->usFontStyle = usFontStyle;
+			pOutput->usFontSize = usFontSize;
+			pOutput->tFontRef = tOpenFont(
+					ucFontNumber, usFontStyle, usFontSize);
 		}
 		bWasTableRow = bIsTableRow;
 
@@ -1098,4 +1171,4 @@ bWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 	vFreeDocument();
 	visdelay_end();
 	return TRUE;
-} /* end of bWord2Text */
+} /* end of bWordDecryptor */
